/*
 * AnyaryOperationAbstraction.hpp
 *
 *  Created on: 20. 8. 2017
 *	  Author: Jan Travnicek
 */

#ifndef _ANYARY_OPERATION_ABSTRACTION_HPP_
#define _ANYARY_OPERATION_ABSTRACTION_HPP_

#include <abstraction/ValueOperationAbstraction.hpp>
#include <alib/memory>
#include <registry/Registry.h>

namespace abstraction {

template < class ReturnType, class ParamType >
class AnyaryOperationAbstraction : public ValueOperationAbstraction < ReturnType > {
protected:
	ext::vector < std::pair < std::shared_ptr < ValueProvider < ParamType > >, bool > > m_params;

private:
	virtual bool attachInput ( const std::shared_ptr < OperationAbstraction > & input, unsigned index, bool move ) override {
		std::shared_ptr < ValueProvider < ParamType > > validData = std::dynamic_pointer_cast < ValueProvider < ParamType > > ( input->getProxyAbstraction ( ) );
		if ( validData ) {
			if ( m_params.size ( ) < index + 1 )
				m_params.resize ( index + 1 );

			m_params [ index ] = std::make_pair ( validData, move );
			return true;
		} else {
			return false;
		}
	}

	virtual bool detachInput ( unsigned index ) override {
		if ( index >= m_params.size ( ) )
			throw std::invalid_argument ( "Parameter index " + ext::to_string ( index ) + " out of bounds.");

		m_params [ index ].first = nullptr;
		m_params [ index ].second = false;

		return true;
	}

public:
	AnyaryOperationAbstraction ( ) {
	}

	virtual bool inputsReady ( ) const override {
		for ( const auto & param : m_params )
			if ( ! ( bool ) param.first || ! param.first->isReady ( ) )
				return false;

		return true;
	}

	virtual bool inputsAttached ( ) const override {
		for ( const auto & param : m_params )
			if ( ! ( bool ) param.first )
				return false;

		return true;
	}

	virtual bool eval ( ) override {
		if ( ! inputsAttached ( ) )
			return false;

		if ( this->cached ( ) )
			return true;

		for ( const std::pair < std::shared_ptr < ValueProvider < ParamType > >, bool > & param : m_params )
			if ( ! param.first->eval ( ) )
				return false;

		return this->run ( );
	}

	virtual unsigned numberOfParams ( ) const override {
		return m_params.size ( );
	}

	virtual ext::type_index getParamTypeIndex ( unsigned ) const override {
		return ext::type_index ( typeid ( ParamType ) );
	}

};

} /* namespace abstraction */

#endif /* _ANYARY_OPERATION_ABSTRACTION_HPP_ */
