/*
 * ValueProvider.hpp
 *
 *  Created on: 11. 7. 2017
 *	  Author: Jan Travnicek
 */

#ifndef _VALUE_PROVIDER_HPP_
#define _VALUE_PROVIDER_HPP_

#include <exception>
#include <alib/utility>

namespace abstraction {

template < class Type >
class ValueProvider {
protected:
	virtual Type & getData ( ) const = 0;

public:
	typedef Type return_type;

	template < class T = Type >
	typename std::enable_if < std::is_copy_constructible < T >::value && std::is_move_constructible < T >::value, T >::type getValue ( bool move ) const {
		if ( move )
			return std::move ( getData ( ) );
		else
			return getData ( );
	}

	template < class T = Type >
	typename std::enable_if < std::is_copy_constructible < T >::value && ! std::is_move_constructible < T >::value, T >::type getValue ( bool move ) const {
		if ( move )
			throw std::domain_error ( "Value not move constructible" );
		else
			return getData ( );
	}

	template < class T = Type >
	typename std::enable_if < ! std::is_copy_constructible < T >::value && std::is_move_constructible < T >::value, T >::type getValue ( bool move ) const {
		if ( move )
			return std::move ( getData ( ) );
		else
			throw std::domain_error ( "Value not copy constructible" );
	}

	template < class T = Type >
	typename std::enable_if < ! std::is_copy_constructible < T >::value && ! std::is_move_constructible < T >::value, T >::type getValue ( bool move ) const {
		if ( move )
			throw std::domain_error ( "Value not move constructible" );
		else
			throw std::domain_error ( "Value not copy constructible" );
	}

	virtual bool eval ( ) = 0;

	virtual bool isReady ( ) const = 0;
};

template < class Type >
class ValueProvider < Type & > {
protected:
	virtual Type & getData ( ) const = 0;

public:
	typedef Type return_type;

	Type & getValueReference ( ) const {
		return getData ( );
	}

	Type & getValue ( bool ) const {
		return getData ( );
	}

	virtual bool eval ( ) = 0;

	virtual bool isReady ( ) const = 0;
};

template < class Type >
class ValueProvider < const Type & > {
protected:
	virtual const Type & getConstData ( ) const = 0;

public:
	typedef Type return_type;

	const Type & getConstValueReference ( ) const {
		return getConstData ( );
	}

	const Type & getValue ( bool ) const {
		return getConstData ( );
	}

	virtual bool eval ( ) = 0;

	virtual bool isReady ( ) const = 0;
};

template < class Type >
class ValueProvider < Type && > {
protected:
	virtual Type & getData ( ) const = 0;

public:
	typedef Type return_type;

	Type && getRValueReference ( ) const {
		return std::move ( getData ( ) );
	}

	Type && getValue ( bool move ) const {
		if ( move )
			return std::move ( getData ( ) );
		else
			throw std::domain_error ( "Value not copy constructible" );
	}

	virtual bool eval ( ) = 0;

	virtual bool isReady ( ) const = 0;
};

template < class Type >
class ValueProvider < const Type && > {
protected:
	virtual const Type & getConstData ( ) const = 0;

public:
	typedef Type return_type;

	const Type && getConstRValueReference ( ) const {
		return std::move ( getConstData ( ) );
	}

	const Type && getValue ( bool move ) const {
		if ( move )
			return std::move ( getConstData ( ) );
		else
			throw std::domain_error ( "Value not copy constructible" );
	}

	virtual bool eval ( ) = 0;

	virtual bool isReady ( ) const = 0;
};

} /* namespace abstraction */

#endif /* _VALUE_PROVIDER_HPP_ */
