/*
 * RankedSymbol.h
 *
 *  Created on: Apr 10, 2013
 *      Author: Stepan Plachy
 */

#ifndef _XML_RANKED_SYMBOL_H_
#define _XML_RANKED_SYMBOL_H_

#include <common/ranked_symbol.hpp>
#include <core/xmlApi.hpp>
#include <primitive/xml/Unsigned.h>

namespace core {

template < typename T, typename R >
struct xmlApi < common::ranked_symbol < T, R > > {
	static common::ranked_symbol < T, R > parse ( ext::deque < sax::Token >::iterator & input );
	static bool first ( const ext::deque < sax::Token >::const_iterator & input );
	static const std::string & xmlTagName ( );
	static void compose ( ext::deque < sax::Token > & output, const common::ranked_symbol < T, R > & data );
};

template < typename T, typename R >
common::ranked_symbol < T, R > xmlApi < common::ranked_symbol < T, R > >::parse ( ext::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, xmlTagName ( ) );
	T data = core::xmlApi < T >::parse ( input );
	R rank = core::xmlApi < R >::parse ( input );
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, xmlTagName ( ) );
	return common::ranked_symbol < T, R > ( std::move ( data ), std::move ( rank ) );
}

template < typename T, typename R >
bool xmlApi < common::ranked_symbol < T, R > >::first ( const ext::deque < sax::Token >::const_iterator & input ) {
	return sax::FromXMLParserHelper::isToken ( input, sax::Token::TokenType::START_ELEMENT, xmlTagName( ) );
}

template < typename T, typename R >
const std::string & xmlApi < common::ranked_symbol < T, R > >::xmlTagName ( ) {
	static std::string xmlTagName = "RankedSymbol";

	return xmlTagName;
}

template < typename T, typename R >
void xmlApi < common::ranked_symbol < T, R > >::compose ( ext::deque < sax::Token > & output, const common::ranked_symbol < T, R > & input ) {
	output.emplace_back ( xmlTagName ( ), sax::Token::TokenType::START_ELEMENT );
	core::xmlApi < T >::compose ( output, input.getSymbol ( ) );
	core::xmlApi < R >::compose ( output, input.getRank ( ) );
	output.emplace_back ( xmlTagName ( ), sax::Token::TokenType::END_ELEMENT );
}

} /* namespace core */

#endif /* _XML_RANKED_SYMBOL_H_ */
