/*
 * ToPostfixPushdownAutomaton.h
 *
 * This file is part of Algorithms library toolkit.
 * Copyright (C) 2017 Jan Travnicek (jan.travnicek@fit.cvut.cz)

 * Algorithms library toolkit is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * Algorithms library toolkit is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with Algorithms library toolkit.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Created on: 13. 3. 2017
 *	  Author: Stepan Plachy
 */

#ifndef AUTOMATON_TO_AUTOMATON_H_
#define AUTOMATON_TO_AUTOMATON_H_

#include <automaton/TA/DFTA.h>
#include <automaton/TA/NFTA.h>
#include <automaton/PDA/NPDA.h>
#include <automaton/PDA/DPDA.h>

namespace automaton {

namespace convert {

/**
 * Converts a finite tree automaton (FTA) to a pushdown automaton (PDA) that reads linearised trees in their postfix notation.
 */
class ToPostfixPushdownAutomaton {
public:
	/**
	 * Performs the conversion of the deterministic FTA to the deterministic PDA
	 * @param dfta Deterministic finite tree automaton to convert
	 * @return (D)PDA equivalent to original finite tree automaton reading linearized postfix tree
	 */
	static automaton::DPDA < > convert ( const automaton::DFTA < > & dfta );

	/**
	 * Performs the conversion of the nondeterministic FTA to the nondeterministic PDA.
	 * @param nfta Nondeterministic finite tree automaton to convert
	 * @return (N)PDA equivalent to original finite tree automaton reading linearized postfix tree
	 */
	static automaton::NPDA < > convert ( const automaton::NFTA < > & nfta );
};

} /* namespace convert */

} /* namespace automaton */

#endif /* AUTOMATON_TO_AUTOMATON_H_ */
