/*
 * NFTADeterminizer.hxx
 *
 *  Created on: 30. 3. 2015
 *	  Author: Stepan Plachy
 */

#include <automaton/TA/NFTA.h>
#include <alib/deque>
#include <alib/algorithm>

namespace automaton {

namespace determinize {

template < class SymbolType, class RankType, class StateType >
ext::set < StateType > getTransitionRightSide ( const NFTA < SymbolType, RankType, StateType > & nfta, const common::ranked_symbol < SymbolType, RankType > & symbol, const ext::vector< ext::set < StateType > > & states ) {
	ext::set < StateType > res;

	for ( const auto & transition : nfta.getTransitions ( ) ) {
		if ( transition.first.first != symbol )
			continue;

		unsigned i = ( unsigned ) symbol.getRank ( );
		for ( ; i > 0; --i )
			if ( ! states [ i - 1 ].count ( transition.first.second [ i - 1 ] ) )
				break;

		if ( i == 0 )
			res.insert ( transition.second.begin ( ), transition.second.end ( ) );
	}

	return res;
}

template < class SymbolType, class RankType, class StateType >
automaton::DFTA < SymbolType, RankType, ext::set < StateType > > Determinize::determinize ( const automaton::NFTA < SymbolType, RankType, StateType > & nfta ) {
	automaton::DFTA < SymbolType, RankType, ext::set < StateType > > res;

	res.setInputAlphabet ( nfta.getInputAlphabet ( ) );
	for ( const auto & state : nfta.getStates ( ) )
		res.addState ( { state } );

	for ( const auto & state : nfta.getFinalStates ( ) )
		res.addFinalState ( { state } );

	ext::deque < ext::set < StateType > > todo;

	for ( const auto & transition : nfta.getTransitions ( ) ) {
		ext::set < StateType > state = transition.second;
		ext::vector < ext::set < StateType > > states;

		for ( const auto & s : transition.first.second )
			states.push_back ( { s } );

		if ( transition.second.size ( ) > 1 && res.addState ( state ) )
			todo.push_back ( state );

		res.addTransition ( transition.first.first, states, state );
	}

	while ( ! todo.empty ( ) ) {
		ext::set < StateType > currentState = todo.front();
		todo.pop_front();

		ext::deque < std::pair < std::pair < common::ranked_symbol < SymbolType, RankType >, ext::vector < ext::set < StateType > > >, ext::set < StateType > > > transitions;
		ext::deque < unsigned > stops;

		for ( const auto & transition : res.getTransitions ( ) ) {
			transitions.push_back ( transition );
			stops.push_back ( 0 );
		}

		while ( ! transitions.empty ( ) ) {
			auto transition = transitions.front ( );
			transitions.pop_front ( );

			unsigned stop = stops.front ( );
			stops.pop_front ( );

			const ext::vector < ext::set < StateType > > & states = transition.first.second;

			for ( unsigned i = stop; i < states.size ( ); ++i ) {
				if ( states [ i ].size ( ) != 1 )
					continue;

				StateType nftaState = * states [ i ].begin ( );
				if ( currentState.count ( nftaState ) ) {
					ext::vector < ext::set < StateType > > newStates = states;
					newStates [ i ] = currentState;

					ext::set < StateType > newNextState = getTransitionRightSide ( nfta, transition.first.first, newStates );
					if ( ! newNextState.empty ( ) ) {
						if ( res.addState ( newNextState ) )
							todo.push_back ( newNextState );

						if ( res.addTransition ( transition.first.first, newStates, newNextState ) && i + 1 != states.size ( ) ) {
							transitions.push_back ( ext::make_pair ( ext::make_pair ( transition.first.first, newStates ), newNextState ) );
							stops.push_back(i + 1);
						}
					}
				}
			}
		}

		for ( const auto & state : currentState ) {
			if ( nfta.getFinalStates ( ).count ( state ) ) {
				res.addFinalState ( currentState );
				break;
			}
		}

	}

	return res;
}

} /* namespace determinize */

} /* namespace automaton */
