/*
 * NFADeterminizer.cpp
 *
 *  Created on: 16. 1. 2014
 *	  Author: Jan Travnicek
 */

#include "common/RHDPDACommon.h"

#include <alphabet/BottomOfTheStackSymbol.h>
#include <automaton/PDA/VisiblyPushdownNPDA.h>
#include <alib/set>

namespace automaton {

namespace determinize {

template < class InputSymbolType, class PushdownStoreSymbolType, class StateType >
automaton::VisiblyPushdownDPDA < InputSymbolType, ext::pair < ext::set < ext::pair < StateType, StateType > >, InputSymbolType >, ext::set < ext::pair < StateType, StateType > > > Determinize::determinize ( const automaton::VisiblyPushdownNPDA < InputSymbolType, PushdownStoreSymbolType, StateType > & n ) {
	using DeterministicStateType = ext::set < ext::pair < StateType, StateType > >;
	using DeterministicPushdownStoreSymbolType = ext::pair < DeterministicStateType, InputSymbolType >;

	DeterministicStateType initialLabel = createIdentity(n.getInitialStates());
	DeterministicPushdownStoreSymbolType bottom = ext::make_pair ( DeterministicStateType { }, alphabet::BottomOfTheStackSymbol::instance < InputSymbolType > ( ) );

	automaton::VisiblyPushdownDPDA < InputSymbolType, DeterministicPushdownStoreSymbolType, DeterministicStateType > d(initialLabel, bottom);

	d.setCallInputAlphabet(n.getCallInputAlphabet());
	d.setLocalInputAlphabet(n.getLocalInputAlphabet());
	d.setReturnInputAlphabet(n.getReturnInputAlphabet());

	for(;;) {
		ext::set<ext::pair<DeterministicStateType, DeterministicPushdownStoreSymbolType>> stateSymbols = existsDirtyStateSymbol(d, n);
		ext::set<DeterministicStateType> states = existsDirtyState(d, n);

		if(stateSymbols.empty() && states.empty()) break;

		for ( const ext::pair < DeterministicStateType, DeterministicPushdownStoreSymbolType > & stateSymbol : stateSymbols ) {
			const DeterministicStateType & state = stateSymbol.first;
			const DeterministicPushdownStoreSymbolType & pdaSymbol = stateSymbol.second;

			for ( const InputSymbolType & input : n.getReturnInputAlphabet ( ) ) {
				if ( pdaSymbol == d.getBottomOfTheStackSymbol ( ) ) {
					DeterministicStateType to = retInitial ( state, input, n );

					d.addState ( to );
					d.addReturnTransition ( state, input, pdaSymbol, std::move ( to ) );
				} else {
					DeterministicStateType to = ret ( state, pdaSymbol, input, n );

					d.addState ( to );
					d.addReturnTransition ( state, input, pdaSymbol, std::move ( to ) );
				}
			}
		}

		for ( const DeterministicStateType & state : states ) {
			for ( const InputSymbolType & input : n.getLocalInputAlphabet ( ) ) {
				DeterministicStateType to = local ( state, input, n );

				d.addState ( to );
				d.addLocalTransition ( state, input, std::move ( to ) );
			}
			for ( const InputSymbolType & input : n.getCallInputAlphabet ( ) ) {
				DeterministicStateType to = call ( state, input, n );
				ext::pair < DeterministicStateType, InputSymbolType > pdaSymbol = ext::make_pair ( state, input );

				d.addState ( to );
				d.addPushdownStoreSymbol ( pdaSymbol );
				d.addCallTransition ( state, input, std::move ( to ), std::move ( pdaSymbol ) );
			}
		}
	}

	const ext::set<StateType>& finalLabels = n.getFinalStates();
	for(const DeterministicStateType & state : d.getStates()) {
		const ext::set<StateType> labels = retrieveDSubSet(state);

		if(!ext::excludes(finalLabels.begin(), finalLabels.end(), labels.begin(), labels.end())) {
			d.addFinalState(state);
		}
	}

	return d;
}

} /* namespace determinize */

} /* namespace automaton */
