/*
 * EpsilonRemoverOutgoing.h
 *
 *  Created on: 16. 1. 2014
 *	  Author: Jan Travnicek
 */

#ifndef EFFICIENT_EPSILON_REMOVER_OUTGOING_H_
#define EFFICIENT_EPSILON_REMOVER_OUTGOING_H_

#include <map>
#include <algorithm>

#include <automaton/FSM/EpsilonNFA.h>
#include <automaton/FSM/MultiInitialStateNFA.h>
#include <automaton/FSM/NFA.h>
#include <automaton/FSM/DFA.h>

#include "../../properties/efficient/AllEpsilonClosure.h"

namespace automaton {

namespace simplify {

namespace efficient {

class EpsilonRemoverOutgoing {
public:
	/**
	 * Computes epsilon closure of a state in epsilon nonfree automaton
	 */
	template < class StateType, class EpsilonType, class SymbolType >
	static automaton::MultiInitialStateNFA < SymbolType, StateType > remove ( const automaton::EpsilonNFA < SymbolType, EpsilonType, StateType > & fsm );
	template < class StateType, class SymbolType >
	static automaton::MultiInitialStateNFA < SymbolType, StateType > remove ( const automaton::MultiInitialStateNFA < SymbolType, StateType > & fsm );
	template < class StateType, class SymbolType >
	static automaton::NFA < SymbolType, StateType > remove ( const automaton::NFA < SymbolType, StateType > & fsm );
	template < class StateType, class SymbolType >
	static automaton::DFA < SymbolType, StateType > remove ( const automaton::DFA < SymbolType, StateType > & fsm );
};

template < class StateType, class SymbolType >
automaton::DFA < SymbolType, StateType > EpsilonRemoverOutgoing::remove ( const automaton::DFA < SymbolType, StateType > & origFSM) {
	return origFSM;
}

template < class StateType, class SymbolType >
automaton::MultiInitialStateNFA < SymbolType, StateType > EpsilonRemoverOutgoing::remove ( const automaton::MultiInitialStateNFA < SymbolType, StateType > & origFSM) {
	return origFSM;
}

template < class StateType, class SymbolType >
automaton::NFA < SymbolType, StateType > EpsilonRemoverOutgoing::remove ( const automaton::NFA < SymbolType, StateType > & origFSM) {
	return origFSM;
}

template < class StateType, class EpsilonType, class SymbolType >
automaton::MultiInitialStateNFA < SymbolType, StateType > EpsilonRemoverOutgoing::remove ( const automaton::EpsilonNFA < SymbolType, EpsilonType, StateType > & origFSM ) {
	automaton::MultiInitialStateNFA < SymbolType, StateType > fsm;
	fsm.setStates ( origFSM.getStates ( ) );
	fsm.setFinalStates ( origFSM.getFinalStates ( ) );
	fsm.setInputAlphabet ( origFSM.getInputAlphabet ( ) );

	ext::map < StateType, ext::set < StateType > > closures = automaton::properties::efficient::AllEpsilonClosure::allEpsilonClosure ( origFSM );

	ext::map < ext::pair < StateType, SymbolType >, ext::set < StateType > > transitions = origFSM.getSymbolTransitions ( );

	for ( const std::pair < const ext::pair < StateType, SymbolType >, ext::set < StateType > > & transition : transitions )
		for ( const StateType & to : transition.second )
			for ( const StateType & toClosure : closures [ to ] )
				fsm.addTransition ( transition.first.first, transition.first.second, toClosure );

	/**
	 * Step 2 from Melichar 2.41
	 */
	const ext::set < StateType > & cl = closures [ origFSM.getInitialState ( ) ];
	fsm.setInitialStates ( cl );

	return fsm;
}

} /* namespace efficient */

} /* namespace simplify */

} /* namespace automaton */

#endif /* EFFICIENT_EPSILON_REMOVER_OUTGOING_H_ */
