#ifndef _CLI_CAST_INTROSPECTION_COMMAND_H_
#define _CLI_CAST_INTROSPECTION_COMMAND_H_

#include <command/Command.h>
#include <environment/Environment.h>

namespace cli {

class CastsIntrospectionCommand : public Command {
	std::unique_ptr < cli::Arg > m_param;
	bool m_from;
	bool m_to;

public:
	CastsIntrospectionCommand (std::unique_ptr < cli::Arg > param, bool from, bool to ) : m_param ( std::move ( param ) ), m_from ( from ), m_to ( to ) {
	}

	void printTypes ( const ext::set < std::string > & types ) const {
		for ( const std::string & type : types )
			common::Streams::out << type << std::endl;
	}

	void printCasts ( const ext::set < ext::pair < std::string, std::string > > & casts ) const {
		for ( const ext::pair < std::string, std::string > & cast : casts )
			common::Streams::out << cast.first << ", " << cast.second << std::endl;
	}

	virtual Command::Result run ( Environment & environment ) const override {
		std::string param;
		if ( m_param != nullptr )
			param = m_param->eval ( environment );

		if ( m_from )
			printTypes ( abstraction::Registry::listCastsFrom ( param ) );

		if ( m_to )
			printTypes ( abstraction::Registry::listCastsTo ( param ) );

		if ( ! m_from && ! m_to )
			printCasts ( abstraction::Registry::listCasts ( ) );

		return Command::Result::OK;
	}
};

} /* namespace cli */

#endif /* _CLI_CAST_INTROSPECTION_COMMAND_H_ */
