#ifndef _CLI_OVERLOADS_INTROSPECTION_COMMAND_H_
#define _CLI_OVERLOADS_INTROSPECTION_COMMAND_H_

#include <command/Command.h>
#include <environment/Environment.h>

namespace cli {

class OverloadsIntrospectionCommand : public Command {
	std::unique_ptr < cli::Arg > m_param;
	ext::vector < std::unique_ptr < cli::Arg > > m_templateParams;

public:
	OverloadsIntrospectionCommand ( std::unique_ptr < cli::Arg > param, ext::vector < std::unique_ptr < cli::Arg > > templateParams ) : m_param ( std::move ( param ) ), m_templateParams ( std::move ( templateParams ) ) {
	}

	virtual Command::Result run ( Environment & environment ) const override {
		std::string param = m_param->eval ( environment );
		ext::vector < std::string > templateParams;
		for ( const std::unique_ptr < cli::Arg > & templateParam : m_templateParams )
			templateParams.push_back ( templateParam->eval ( environment ) );

		ext::set < ext::tuple < abstraction::AlgorithmCategories::AlgorithmCategory, ext::pair < std::string, ext::set < abstraction::ParamQualifiers::ParamQualifier > >, ext::vector < ext::tuple < std::string, ext::set < abstraction::ParamQualifiers::ParamQualifier >, std::string > > > > overloads = abstraction::Registry::listOverloads ( param, templateParams );
		for ( const ext::tuple < abstraction::AlgorithmCategories::AlgorithmCategory, ext::pair < std::string, ext::set < abstraction::ParamQualifiers::ParamQualifier > >, ext::vector < ext::tuple < std::string, ext::set < abstraction::ParamQualifiers::ParamQualifier >, std::string > > > & overload : overloads ) {
			common::Streams::out << std::get < 0 > ( overload ) << ": ";
			{
				const ext::pair < std::string, ext::set < abstraction::ParamQualifiers::ParamQualifier > > & result = std::get < 1 > ( overload );

				if ( std::get < 1 > ( result ).count ( abstraction::ParamQualifiers::ParamQualifier::CONST ) )
					common::Streams::out << "const ";

				common::Streams::out << std::get < 0 > ( result );

				if ( std::get < 1 > ( result ).count ( abstraction::ParamQualifiers::ParamQualifier::LREF ) )
					common::Streams::out << " &";

				if ( std::get < 1 > ( result ).count ( abstraction::ParamQualifiers::ParamQualifier::RREF ) )
					common::Streams::out << " &&";
			}
			common::Streams::out << " (";
			bool first = true;
			for ( const ext::tuple < std::string, ext::set < abstraction::ParamQualifiers::ParamQualifier >, std::string > & parameter : std::get < 2 > ( overload ) ) {
				if ( first )
					first = false;
				else
					common::Streams::out << ",";

				if ( std::get < 1 > ( parameter ).count ( abstraction::ParamQualifiers::ParamQualifier::CONST ) )
					common::Streams::out << " const ";
				else
					common::Streams::out << " ";

				common::Streams::out << std::get < 0 > ( parameter );

				if ( std::get < 1 > ( parameter ).count ( abstraction::ParamQualifiers::ParamQualifier::LREF ) )
					common::Streams::out << " &";

				if ( std::get < 1 > ( parameter ).count ( abstraction::ParamQualifiers::ParamQualifier::RREF ) )
					common::Streams::out << " &&";

				common::Streams::out << " " << std::get < 2 > ( parameter );
			}
			common::Streams::out << " )" << std::endl;
		}

		return Command::Result::OK;
	}
};

} /* namespace cli */

#endif /* _CLI_OVERLOADS_INTROSPECTION_COMMAND_H_ */
