/*
 * ranked_symbol.h
 *
 *  Created on: Apr 10, 2013
 *      Author: Stepan Plachy
 */

#ifndef RAW_RANKED_SYMBOL_H_
#define RAW_RANKED_SYMBOL_H_

#include <alib/tuple>

#include <core/normalize.hpp>
#include <sax/FromXMLParserHelper.h>

#include <common/DefaultSymbolType.h>
#include <common/DefaultRankType.h>

namespace common {

/**
 * Represents symbol in an std.
 */
template < class SymbolType = DefaultSymbolType, class RankType = DefaultRankType >
class ranked_symbol {
protected:
	SymbolType m_symbol;
	RankType m_rank;

public:
	/**
	 * Creates new symbol with given name and rank.
	 * @param symbol name of the symbol
	 * @param rank of the symbol
	 */
	explicit ranked_symbol ( SymbolType symbol, RankType rank );

	/**
	 * @return name of the symbol
	 */
	const SymbolType & getSymbol ( ) const &;

	/**
	 * @return name of the symbol
	 */
	SymbolType && getSymbol ( ) &&;

	/**
	 * @return rank of the symbol
	 */
	const RankType & getRank ( ) const &;

	/**
	 * @return rank of the symbol
	 */
	RankType && getRank ( ) &&;

	int compare ( const ranked_symbol & other ) const;

	explicit operator std::string ( ) const;

	bool operator < ( const ranked_symbol & other ) const {
		return compare ( other ) < 0;
	}

	bool operator > ( const ranked_symbol & other ) const {
		return compare ( other ) > 0;
	}

	bool operator <= ( const ranked_symbol & other ) const {
		return compare ( other ) <= 0;
	}

	bool operator >= ( const ranked_symbol & other ) const {
		return compare ( other ) >= 0;
	}

	bool operator == ( const ranked_symbol & other ) const {
		return compare ( other ) == 0;
	}

	bool operator != ( const ranked_symbol & other ) const {
		return compare ( other ) != 0;
	}

	ranked_symbol < SymbolType, RankType > & operator ++ ( ) {
		++ m_symbol;

		return *this;
	}
};

template < class SymbolType, class RankType >
ranked_symbol < SymbolType, RankType >::ranked_symbol(SymbolType symbol, RankType rank) : m_symbol(std::move(symbol)), m_rank(std::move(rank)) {

}

template < class SymbolType, class RankType >
const SymbolType & ranked_symbol < SymbolType, RankType >::getSymbol ( ) const & {
	return m_symbol;
}

template < class SymbolType, class RankType >
SymbolType && ranked_symbol < SymbolType, RankType >::getSymbol ( ) && {
	return std::move ( m_symbol );
}

template < class SymbolType, class RankType >
const RankType & ranked_symbol < SymbolType, RankType >::getRank ( ) const & {
	return m_rank;
}

template < class SymbolType, class RankType >
RankType && ranked_symbol < SymbolType, RankType >::getRank ( ) && {
	return std::move ( m_rank );
}

template < class SymbolType, class RankType >
int ranked_symbol < SymbolType, RankType >::compare(const ranked_symbol& other) const {
	auto first = ext::tie ( m_symbol, m_rank );
	auto second = ext::tie ( other.m_symbol, other.m_rank );

	static ext::compare<decltype(first)> comp;
	return comp(first, second);
}

template < class SymbolType, class RankType >
ranked_symbol < SymbolType, RankType >::operator std::string () const {
	return ext::to_string ( m_symbol ) + "_" + ext::to_string ( m_rank );
}

template < class SymbolType, class RankType >
std::ostream & operator << ( std::ostream & out, const common::ranked_symbol < SymbolType, RankType > & symbol ) {
	out << "(ranked_symbol " << symbol.getSymbol ( ) << " #" << symbol.getRank ( ) << ")";
	return out;
}

} /* namespace common */

namespace ext {

template < class T, class R >
struct compare < common::ranked_symbol < T, R > > {
	int operator ()( const common::ranked_symbol < T, R > & first, const common::ranked_symbol < T, R > & second ) const {
		return first.compare ( second );
	}

};

} /* namespace ext */

#endif /* RAW_RANKED_SYMBOL_H_ */
