#ifndef _ENVIRONMENT_H_
#define _ENVIRONMENT_H_

#include <alib/string>
#include <alib/map>
#include <alib/memory>
#include <alib/typeinfo>

#include <abstraction/OperationAbstraction.hpp>
#include <exception/CommonException.h>

#include <abstraction/ImmediateValueAbstraction.hpp>

namespace cli {

class Environment {
	ext::map < std::string, std::string > m_bindings;
	ext::map < std::string, std::shared_ptr < abstraction::OperationAbstraction > > m_variables;

	std::unique_ptr < Environment > m_upper;

	std::shared_ptr < abstraction::OperationAbstraction > getVariableInt ( const std::string & name ) {
		auto res = m_variables.find ( name );

		if ( res != m_variables.end ( ) )
			return res->second;

		if ( ( bool ) m_upper )
			return m_upper->getVariable ( name );
		else
			throw exception::CommonException ( "Variable of name " + name + " not found." );
	}

	void setVariableInt ( std::string name, std::shared_ptr < abstraction::OperationAbstraction > value ) {
		m_variables [ std::move ( name ) ] = std::move ( value );
	}

public:
	Environment ( ) {
	}

	Environment ( std::unique_ptr < Environment > upper ) : m_upper ( std::move ( upper ) ) {
	}

	std::string getBinding ( const std::string & name ) const {
		auto res = m_bindings.find ( name );

		if ( res != m_bindings.end ( ) )
			return res->second;

		if ( ( bool ) m_upper )
			return m_upper->getBinding ( name );
		else
			throw exception::CommonException ( "Binded value of name " + name + " not found." );
	}

	void setBinding ( std::string name, std::string value ) {
		m_bindings [ std::move ( name ) ] = std::move ( value );
	}

	bool clearBinding ( const std::string & name ) {
		return m_bindings.erase ( name );
	}

	std::shared_ptr < abstraction::OperationAbstraction > getVariable ( const std::string & name ) {
		return getVariableInt ( name );
	}

	template < class T >
	const T & getVariable ( const std::string & name ) {
		std::shared_ptr < abstraction::ValueProvider < const T & > > ptr = std::dynamic_pointer_cast < abstraction::ValueProvider < const T & > > ( getVariableInt ( name ) );
		if ( ! ptr )
			throw exception::CommonException ( "Invalid variable type. Requested: " + ext::to_string < T > ( ) + ", actual : " + getVariableInt ( name )->getRuntimeReturnType ( ) );
		return ptr->getConstValueReference ( );
	}

	void setVariable ( std::string name, std::shared_ptr < abstraction::OperationAbstraction > value ) {
		setVariableInt ( std::move ( name ), std::move ( value ) );
	}

	template < class T >
	void setVariable ( std::string name, const T & value ) {
		setVariableInt ( name, std::make_shared < abstraction::ImmediateValueAbstraction < T > > ( value ) );
	}

	bool clearVariable ( const std::string & name ) {
		return m_variables.erase ( name );
	}

};

} /* namespace cli */

#endif /* _ENVIRONMENT_H_ */
