/*
 * tuple.hpp
 *
 * Created on: Apr 1, 2013
 * Author: Jan Travnicek
 */

#ifndef __TUPLE_COMMON_HPP_
#define __TUPLE_COMMON_HPP_

#include <tuple>

namespace ext {

template < class ... Types >
class tuple_size;

template < class T >
class tuple_size < T & > : public std::integral_constant < std::size_t, ext::tuple_size < T >::value > { };

template < class T >
class tuple_size < T && > : public std::integral_constant < std::size_t, ext::tuple_size < T >::value > { };

template < class T >
class tuple_size < const T > : public std::integral_constant < std::size_t, ext::tuple_size < T >::value > { };

template < class T >
class tuple_size< volatile T > : public std::integral_constant < std::size_t, ext::tuple_size < T >::value > { };

template< class T >
class tuple_size < const volatile T > : public std::integral_constant < std::size_t, tuple_size < T >::value > { };



template < std::size_t I, class T >
struct tuple_element;

template < std::size_t I, class T >
struct tuple_element < I, const T > {
	typedef typename std::add_const < typename ext::tuple_element < I, T >::type >::type type;
};

template < std::size_t I, class T >
struct tuple_element< I, volatile T > {
	typedef typename std::add_volatile < typename ext::tuple_element < I, T >::type >::type type;
};

template < std::size_t I, class T >
struct tuple_element < I, const volatile T > {
	typedef typename std::add_cv < typename ext::tuple_element < I, T >::type >::type type;
};



template < unsigned I >
struct call_on_nth_helper {
	template < class Result, class Tuple, class Callable >
	static Result call_on_nth_fn ( Tuple && t, unsigned index, Callable callback ) {
		if ( index == 0 )
			return callback ( std::forward < Tuple && > ( t ).template get < ext::tuple_size < Tuple >::value - I > ( ) );
		else
			return call_on_nth_helper < I - 1 >::template call_on_nth_fn < Result > ( std::forward < Tuple && > ( t ), index - 1, callback );
	}
};

template < >
struct call_on_nth_helper < 0 > {
	template < class Result, class Tuple, class Callable >
	static Result call_on_nth_fn ( Tuple &&, unsigned, Callable ) {
		throw std::out_of_range ( "Not enough elements in tuple." );
	}
};

template < class Result, class Tuple, class Callable >
Result call_on_nth ( Tuple && t, unsigned index, Callable callback ) {
	return call_on_nth_helper < ext::tuple_size < Tuple >::value >::template call_on_nth_fn < Result > ( std::forward < Tuple && > ( t ), index, callback );
}



template < unsigned I >
struct foreach_helper {
	template < class Tuple, class Callable >
	static void foreach_fn ( Tuple && t, Callable callback ) {
		foreach_helper < I - 1 >::foreach_fn ( std::forward < Tuple && > ( t ), callback );
		callback ( std::forward < Tuple && > ( t ).template get < I - 1 > ( ) );
	}
};

template < >
struct foreach_helper < 0 > {
	template < class Tuple, class Callable >
	static void foreach_fn ( Tuple &&, Callable ) {
	}
};

template < class Tuple, class Callable >
void foreach ( Tuple && t, Callable callback ) {
	return foreach_helper < ext::tuple_size < Tuple >::value >::foreach_fn ( std::forward < Tuple && > ( t ), callback );
}



template < int I >
struct compareTupleHelper {
	template < class Tuple >
	static int compHelp ( const Tuple & t1, const Tuple & t2 ) {
		int res = compareTupleHelper < I - 1 >::compHelp ( t1, t2 );

		if ( res != 0 ) return res;

		static compare < typename std::decay < typename ext::tuple_element < I, Tuple >::type >::type > comp;
		return comp ( t1.template get < I > ( ), t2.template get < I > ( ) );
	}

};

template < >
struct compareTupleHelper < 0 > {
	template < class Tuple >
	static int compHelp ( const Tuple & t1, const Tuple & t2 ) {
		static compare < typename std::decay < typename ext::tuple_element < 0, Tuple >::type >::type > comp;
		return comp ( t1.template get < 0 > ( ), t2.template get < 0 > ( ) );
	}

};



template < class Tuple, class Callable >
bool all_of ( const Tuple & t, Callable callback ) {
	bool res = true;

	auto aggregateCallback = [ & ] ( const auto & arg0 ) {
		res &= callback ( arg0 );
	};

	ext::foreach ( t, aggregateCallback  );
	return res;
}

} /* namespace ext */

#endif /* __TUPLE_COMMON_HPP_ */
