/*
 * GenerateUpToLength.h
 *
 *  Created on: 22. 3. 2014
 *	  Author: Jan Travnicek
 */

#ifndef GRAMMAR_GENERATE_UP_TO_LENGTH_H_
#define GRAMMAR_GENERATE_UP_TO_LENGTH_H_

#include <grammar/Grammar.h>
#include <string/LinearString.h>
#include <alib/set>
#include <alib/deque>

#include <grammar/ContextFree/EpsilonFreeCFG.h>
#include <grammar/ContextFree/GNF.h>
#include <grammar/ContextFree/CNF.h>
#include <grammar/Regular/LeftRG.h>
#include <grammar/Regular/RightRG.h>

#include <grammar/RawRules.h>

namespace grammar {

namespace generate {

/**
 * Implements algorithms from Melichar, chapter 3.3
 */
class GenerateUpToLength {
public:
	template < class T, class TerminalSymbolType = typename grammar::TerminalSymbolTypeOfGrammar < T >, class NontermimnalSymbolType = typename grammar::NonterminalSymbolTypeOfGrammar < T > >
	static ext::set < string::LinearString < TerminalSymbolType > > generate ( const T & grammar, unsigned length );
};

template < class T, class TerminalSymbolType, class NonterminalSymbolType >
ext::set < string::LinearString < TerminalSymbolType > > GenerateUpToLength::generate ( const T & grammar, unsigned length ) {
	ext::set < string::LinearString < TerminalSymbolType > > res;

	ext::map < NonterminalSymbolType, ext::set < ext::vector < ext::variant < TerminalSymbolType, NonterminalSymbolType > > > > rules = grammar::RawRules::getRawRules ( grammar );
	if ( grammar.getGeneratesEpsilon ( ) ) {
		res.insert ( string::LinearString < TerminalSymbolType > { } );
		rules [ grammar.getInitialSymbol ( ) ].erase ( ext::vector < ext::variant < TerminalSymbolType, NonterminalSymbolType > > { } );
	}

	ext::deque < std::pair < ext::vector < TerminalSymbolType >, ext::vector < ext::variant < TerminalSymbolType, NonterminalSymbolType > > > > data;
	data.push_back ( std::make_pair ( ext::vector < TerminalSymbolType > { }, ext::vector < ext::variant < TerminalSymbolType, NonterminalSymbolType > > { ext::variant < TerminalSymbolType, NonterminalSymbolType > ( grammar.getInitialSymbol ( ) ) } ) );

	while ( ! data.empty ( ) ) {
		std::pair < ext::vector < TerminalSymbolType >, ext::vector < ext::variant < TerminalSymbolType, NonterminalSymbolType > > > item = std::move ( data.back ( ) );
		data.pop_back ( );

		ext::variant < TerminalSymbolType, NonterminalSymbolType > top = std::move ( item.second.back ( ) );
		item.second.pop_back ( );
		if ( ! top.template is < NonterminalSymbolType > ( ) ) // maybe not needed
			continue;
		auto rule = rules.find ( top.template get < NonterminalSymbolType > ( ) );
		if ( rule == rules.end ( ) )
			continue;

		for ( const ext::vector < ext::variant < TerminalSymbolType, NonterminalSymbolType > > & rhs : rule->second ) {
			if ( item.first.size ( ) + item.second.size ( ) + rhs.size ( ) > length ) continue;

			ext::vector < ext::variant < TerminalSymbolType, NonterminalSymbolType > > newStack ( item.second );
			ext::vector < TerminalSymbolType > newString ( item.first );
			newStack.insert ( newStack.end ( ), rhs.rbegin ( ), rhs.rend ( ) );

			while ( ! newStack.empty ( ) && grammar.getTerminalAlphabet ( ).count ( newStack.back ( ) ) ) {
				newString.push_back ( std::move ( newStack.back ( ).template get < TerminalSymbolType > ( ) ) );
				newStack.pop_back ( );
			}

			if ( newStack.empty ( ) ) {
				res.insert ( string::LinearString < TerminalSymbolType > ( newString ) );
			} else {
				data.push_back ( std::make_pair ( std::move ( newString ), std::move ( newStack ) ) );
			}
		}
	}

	return res;
}

} /* namespace generate */

} /* namespace grammar */

#endif /* GRAMMAR_GENERATE_UP_TO_LENGTH_H_ */
