/*
 * Trim.h
 *
 *  Created on: 22. 3. 2014
 *	  Author: Tomas Pecka
 */

#ifndef GRAMMAR_TRIM_H_
#define GRAMMAR_TRIM_H_

#include <grammar/ContextFree/CFG.h>
#include <grammar/ContextFree/EpsilonFreeCFG.h>
#include <grammar/ContextFree/GNF.h>
#include <grammar/ContextFree/CNF.h>
#include <grammar/ContextFree/LG.h>
#include <grammar/Regular/LeftLG.h>
#include <grammar/Regular/LeftRG.h>
#include <grammar/Regular/RightLG.h>
#include <grammar/Regular/RightRG.h>

#include "UnreachableSymbolsRemover.h"
#include "UnproductiveSymbolsRemover.h"

namespace grammar {

namespace simplify {

/**
 * Implements algorithms from Melichar, chapter 3.3
 */
class Trim {
public:
	/**
	 * Removes unproductive and useless symbols - Melichar 3.12
	 */
	template<class T>
	static T trim( const T & grammar );
};

template<class T>
T Trim::trim( const T & grammar ) {
	return grammar::simplify::UnreachableSymbolsRemover::remove( grammar::simplify::UnproductiveSymbolsRemover::remove( grammar ) );
}

} /* namespace simplify */

} /* namespace grammar */

#endif /* GRAMMAR_TRIM_H_ */
