#ifndef _ALGO_REGISTRATION_HPP_
#define _ALGO_REGISTRATION_HPP_

#include <registry/AlgorithmRegistry.hpp>

#include <registration/NormalizationRegistration.hpp>

namespace registration {

class AlgoRegister {
protected:
	template < size_t ParameterTypesNumber, class ... ParamNames, typename std::enable_if < sizeof ... ( ParamNames ) <= ParameterTypesNumber >::type * = nullptr >
	static std::array < std::string, ParameterTypesNumber > generateNames ( ParamNames ... paramNames ) {
		std::array < std::string, ParameterTypesNumber > parameterNames = { { paramNames ... } };
		for ( unsigned i = sizeof ... ( ParamNames ); i < ParameterTypesNumber; ++i )
			parameterNames [ i ] = "arg" + ext::to_string ( i );

		return parameterNames;
	}
};

template < class Algorithm, class ReturnType, class ... ParameterTypes >
class AbstractRegister : public AlgoRegister {
public:
	template < class ... ParamNames >
	AbstractRegister ( ReturnType ( * callback ) ( ParameterTypes ... ), abstraction::AlgorithmCategories::AlgorithmCategory category, ParamNames ... paramNames ) {
		registration::NormalizationRegister < ReturnType > ( );

		std::array < std::string, sizeof ... ( ParameterTypes ) > parameterNames = generateNames < sizeof ... ( ParameterTypes ) > ( paramNames ... );

		abstraction::AlgorithmRegistry::registerAlgorithm < Algorithm, ReturnType, ParameterTypes ... > ( callback, category, std::move ( parameterNames ) );
	}

	template < class ... ParamNames >
	AbstractRegister ( ReturnType ( * callback ) ( ParameterTypes ... ) ) : AbstractRegister ( callback, abstraction::AlgorithmCategories::AlgorithmCategory::DEFAULT ) {
	}
};

template < class Algorithm, class ReturnType, class ... ParameterTypes >
class WrapperRegister : public AlgoRegister {
public:
	template < class ... ParamNames >
	WrapperRegister ( std::shared_ptr < abstraction::OperationAbstraction > ( * callback ) ( ParameterTypes ... ), ParamNames ... paramNames ) {
		std::array < std::string, sizeof ... ( ParameterTypes ) > parameterNames = generateNames < sizeof ... ( ParameterTypes ) > ( paramNames ... );

		abstraction::AlgorithmRegistry::registerWrapper < Algorithm, ReturnType, ParameterTypes ... > ( callback, std::move ( parameterNames ) );
	}

};

template < class Algorithm, class ReturnType, class ObjectType, class ... ParameterTypes >
class MethodRegister : public AlgoRegister {
public:
	template < class ... ParamNames >
	MethodRegister ( ReturnType ( ObjectType::* callback ) ( ParameterTypes ... ), std::string methodName, ParamNames ... paramNames ) {
		registration::NormalizationRegister < ReturnType > ( );

		std::array < std::string, sizeof ... ( ParameterTypes ) > parameterNames = generateNames < sizeof ... ( ParameterTypes ) > ( paramNames ... );

		abstraction::AlgorithmRegistry::registerMethod < Algorithm > ( callback, methodName, std::move ( parameterNames ) );
	}

};

} /* namespace registration */

#endif // _ALGO_REGISTRATION_HPP_
