#ifndef _RAW_REGISTRATION_HPP_
#define _RAW_REGISTRATION_HPP_

#include <core/xmlApi.hpp>

#include <registry/RawReaderRegistry.hpp>
#include <registry/RawWriterRegistry.hpp>

#include <registry/AlgorithmRegistry.hpp>

namespace raw {

template < class Type >
class Parse {
public:
	static std::shared_ptr < abstraction::OperationAbstraction > abstractionFromString ( const std::string & ) {
		return abstraction::RawReaderRegistry::getAbstraction ( ext::to_string < Type > ( ) );
	}

};

class Compose {
public:
	template < class Type >
	static std::shared_ptr < abstraction::OperationAbstraction > abstractionFromType ( const Type & ) {
		return abstraction::RawWriterRegistry::getAbstraction ( ext::to_string < Type > ( ) );
	}

};

} /* namespace raw */

namespace registration {

template < class Type >
class RawReaderRegister {
public:
	RawReaderRegister ( ) {
		abstraction::RawReaderRegistry::registerRawReader < Type > ( );
		abstraction::AlgorithmRegistry::registerWrapper < raw::Parse < Type >, abstraction::UnspecifiedType, const std::string & > ( raw::Parse < Type >::abstractionFromString, std::array < std::string, 1 > { } );
	}
};

template < class Type >
class RawWriterRegister {
public:
	RawWriterRegister ( ) {
		abstraction::RawWriterRegistry::registerRawWriter < Type > ( );
		abstraction::AlgorithmRegistry::registerWrapper < raw::Compose, std::string, const Type & > ( raw::Compose::abstractionFromType, std::array < std::string, 1 > { } );
	}
};

} /* namespace registration */

#endif // _RAW_REGISTRATION_HPP_
