#ifndef _STRING_REGISTRATION_HPP_
#define _STRING_REGISTRATION_HPP_

#include <core/xmlApi.hpp>

#include <registry/StringWriterRegistry.hpp>
#include <registry/StringReaderRegistry.hpp>

#include <registry/AlgorithmRegistry.hpp>

namespace string {

template < class Group >
class Parse {
public:
	static std::shared_ptr < abstraction::OperationAbstraction > abstractionFromString ( std::string && data ) {
		return abstraction::StringReaderRegistry::getAbstraction ( ext::to_string < Group > ( ), data );
	}

};

class Compose {
public:
	template < class Type >
	static std::shared_ptr < abstraction::OperationAbstraction > abstractionFromType ( const Type & ) {
		return abstraction::StringWriterRegistry::getAbstraction ( ext::to_string < Type > ( ) );
	}

};

} /* namespace string */

namespace registration {

template < class Group, class Type >
class StringReaderRegister {
public:
	StringReaderRegister ( ) {
		abstraction::StringReaderRegistry::registerStringReader < Group, Type > ( );
		abstraction::AlgorithmRegistry::registerWrapper < string::Parse < Group >, abstraction::UnspecifiedType, std::string && > ( string::Parse < Group >::abstractionFromString, std::array < std::string, 1 > { { "arg0" } } );
	}
};

template < class Type >
class StringWriterRegister {
public:
	StringWriterRegister ( ) {
		abstraction::StringWriterRegistry::registerStringWriter < Type > ( );
		abstraction::AlgorithmRegistry::registerWrapper < string::Compose, std::string, const Type & > ( string::Compose::abstractionFromType, std::array < std::string, 1 > { { "arg0" } } );
	}
};

template < class Group, class Type >
class StringReaderRegisterTypeInGroup {
public:
	StringReaderRegisterTypeInGroup ( ) {
		core::stringApi < Group >::template registerStringReader < Type > ( );
	}
};

template < class Group, class Type >
class StringWriterRegisterTypeInGroup {
public:
	StringWriterRegisterTypeInGroup ( ) {
		core::stringApi < Group >::template registerStringWriter < Type > ( );
	}
};

} /* namespace registration */

#endif // _STRING_REGISTRATION_HPP_
