/*
 * ImmediateRegistry.hpp
 *
 *  Created on: 11. 7. 2017
 *	  Author: Jan Travnicek
 */

#ifndef _IMMEDIATE_REGISTRY_HPP_
#define _IMMEDIATE_REGISTRY_HPP_

#include <alib/memory>
#include <alib/string>
#include <alib/map>
#include <alib/typeinfo>

#include <exception>
#include <abstraction/OperationAbstraction.hpp>

namespace abstraction {

class ImmediateRegistry {
	class Entry {
	public:
		virtual std::shared_ptr < abstraction::OperationAbstraction > getAbstraction ( std::string value ) const = 0;

		virtual ~Entry ( ) {
		}
	};

	template < class Result >
	class EntryImpl : public Entry {
	public:
		EntryImpl ( ) {
		}

		virtual ~EntryImpl ( ) {
		}

		virtual std::shared_ptr < abstraction::OperationAbstraction > getAbstraction ( std::string value ) const override;
	};

	static ext::map < std::string, std::unique_ptr < Entry > > & getEntries ( ) {
		static ext::map < std::string, std::unique_ptr < Entry > > fileWriters;
		return fileWriters;
	}

public:
	template < class ResultType >
	static void registerImmediate ( std::string result ) {
		if ( ! getEntries ( ).insert ( std::make_pair ( result, std::unique_ptr < Entry > ( new EntryImpl < ResultType > ( ) ) ) ).second )
			throw std::invalid_argument ( "Entry " + result + " already registered." );
	}

	template < class ResultType >
	static void registerImmediate ( ) {
		std::string result = ext::to_string < ResultType > ( );
		registerImmediate < ResultType > ( std::move ( result ) );
	}

	static std::shared_ptr < abstraction::OperationAbstraction > getAbstraction ( const std::string & result, std::string value );
};

} /* namespace abstraction */

#include <abstraction/ImmediateValueAbstraction.hpp>

namespace abstraction {

template < class Result >
std::shared_ptr < abstraction::OperationAbstraction > ImmediateRegistry::EntryImpl < Result >::getAbstraction ( std::string value ) const {
	return std::make_shared < ImmediateValueAbstraction < Result > > ( ext::from_string < Result > ( value ) );
}

} /* namespace abstraction */

#endif /* _IMMEDIATE_REGISTRY_HPP_ */
