/*
 * RawReaderRegistry.hpp
 *
 *  Created on: 11. 7. 2017
 *	  Author: Jan Travnicek
 */

#ifndef _RAW_READER_REGISTRY_HPP_
#define _RAW_READER_REGISTRY_HPP_

#include <alib/memory>
#include <alib/string>
#include <alib/map>

#include <exception/CommonException.h>
#include <abstraction/OperationAbstraction.hpp>
#include <factory/RawDataFactory.hpp>

namespace abstraction {

class RawReaderRegistry {
	class Entry {
	public:
		virtual std::shared_ptr < abstraction::OperationAbstraction > getAbstraction ( ) const = 0;

		virtual ~Entry ( ) {
		}
	};

	template < class Return >
	class EntryImpl : public Entry {
	public:
		virtual std::shared_ptr < abstraction::OperationAbstraction > getAbstraction ( ) const override;

		virtual ~EntryImpl ( ) {
		}
	};

	static ext::map < std::string, std::unique_ptr < Entry > > & getEntries ( ) {
		static ext::map < std::string, std::unique_ptr < Entry > > readers;
		return readers;
	}

public:
	template < class ReturnType >
	static void registerRawReader ( std::string type ) {
		getEntries ( ).insert ( std::make_pair ( std::move ( type ), std::unique_ptr < Entry > ( new EntryImpl < ReturnType > ( ) ) ) );
	}

	template < class ReturnType >
	static void registerRawReader ( ) {
		std::string type = ext::to_string < ReturnType > ( );
		registerRawReader < ReturnType > ( std::move ( type ) );
	}

	static std::shared_ptr < abstraction::OperationAbstraction > getAbstraction ( const std::string & group );
};

} /* namespace abstraction */

#include <abstraction/RawReaderAbstraction.hpp>

namespace abstraction {

template < class Return >
std::shared_ptr < abstraction::OperationAbstraction > RawReaderRegistry::EntryImpl < Return >::getAbstraction ( ) const {
	return std::make_shared < abstraction::RawReaderAbstraction < Return > > ( );
}

} /* namespace abstraction */

#endif /* _RAW_READER_REGISTRY_HPP_ */
