/*
 * StringReaderRegistry.hpp
 *
 *  Created on: 11. 7. 2017
 *	  Author: Jan Travnicek
 */

#ifndef _STRING_READER_REGISTRY_HPP_
#define _STRING_READER_REGISTRY_HPP_

#include <alib/memory>
#include <alib/string>
#include <alib/map>

#include <exception/CommonException.h>
#include <abstraction/OperationAbstraction.hpp>
#include <factory/StringDataFactory.hpp>

namespace abstraction {

class StringReaderRegistry {
	class Entry {
	public:
		virtual std::shared_ptr < abstraction::OperationAbstraction > getAbstraction ( ) const = 0;

		virtual ~Entry ( ) {
		}
	};

	template < class Return >
	class EntryImpl : public Entry {
	public:
		EntryImpl ( ) {
		}

		virtual ~EntryImpl ( ) {
		}

		virtual std::shared_ptr < abstraction::OperationAbstraction > getAbstraction ( ) const override;
	};

	static ext::map < std::string, ext::deque < std::pair < std::function < bool ( std::istream & ) >, std::unique_ptr < Entry > > > > & getEntries ( ) {
		static ext::map < std::string, ext::deque < std::pair < std::function < bool ( std::istream & ) >, std::unique_ptr < Entry > > > > readers;
		return readers;
	}

public:
	template < class Group, class ReturnType >
	static void registerStringReader ( ) {
		getEntries ( ) [ ext::to_string < Group > ( ) ].push_back ( std::make_pair ( core::stringApi < ReturnType >::first, std::unique_ptr < Entry > ( new EntryImpl < ReturnType > ( ) ) ) );
	}

	static std::shared_ptr < abstraction::OperationAbstraction > getAbstraction ( const std::string & group, const std::string & data );
};

} /* namespace abstraction */

#include <abstraction/StringReaderAbstraction.hpp>

namespace abstraction {

template < class Return >
std::shared_ptr < abstraction::OperationAbstraction > StringReaderRegistry::EntryImpl < Return >::getAbstraction ( ) const {
	return std::make_shared < abstraction::StringReaderAbstraction < Return > > ( );
}

} /* namespace abstraction */

#endif /* _STRING_READER_REGISTRY_HPP_ */
