/*
 * ValuePrinterRegistry.hpp
 *
 *  Created on: 11. 7. 2017
 *	  Author: Jan Travnicek
 */

#ifndef _VALUE_PRINTER_REGISTRY_HPP_
#define _VALUE_PRINTER_REGISTRY_HPP_

#include <alib/memory>
#include <alib/string>
#include <alib/map>
#include <alib/typeinfo>

#include <abstraction/OperationAbstraction.hpp>

namespace abstraction {

class ValuePrinterRegistry {
	class Entry {
	public:
		virtual std::shared_ptr < abstraction::OperationAbstraction > getAbstraction ( std::ostream & os ) const = 0;

		virtual ~Entry ( ) {
		}

	};

	template < class Param >
	class EntryImpl : public Entry {
	public:
		EntryImpl ( ) {
		}

		virtual ~EntryImpl ( ) {
		}

		virtual std::shared_ptr < abstraction::OperationAbstraction > getAbstraction ( std::ostream & os ) const override;
	};

	static ext::map < std::string, std::unique_ptr < Entry > > & getEntries ( ) {
		static ext::map < std::string, std::unique_ptr < Entry > > fileWriters;
		return fileWriters;
	}

public:
	template < class ParamType >
	static void registerValuePrinter ( std::string param ) {
		getEntries ( ).insert ( std::make_pair ( param, std::unique_ptr < Entry > ( new EntryImpl < ParamType > ( ) ) ) );
	}

	template < class ParamType >
	static void registerValuePrinter ( ) {
		std::string param = ext::to_string < ParamType > ( );
		registerValuePrinter < ParamType > ( std::move ( param ) );
	}

	static std::shared_ptr < abstraction::OperationAbstraction > getAbstraction ( const std::string & param, std::ostream & os );
};

} /* namespace abstraction */

#include <abstraction/ValuePrinterAbstraction.hpp>

namespace abstraction {

template < class Param >
std::shared_ptr < abstraction::OperationAbstraction > ValuePrinterRegistry::EntryImpl < Param >::getAbstraction ( std::ostream & os ) const {
	return std::make_shared < abstraction::ValuePrinterAbstraction < const Param & > > ( os );
}

template < >
std::shared_ptr < abstraction::OperationAbstraction > ValuePrinterRegistry::EntryImpl < void >::getAbstraction ( std::ostream & ) const;

} /* namespace abstraction */

#endif /* _VALUE_PRINTER_REGISTRY_HPP_ */
