/*
 * ToPostfixPushdownAutomatonGlushkov.h
 *
 *  Created on: 26. 7. 2017
 *	  Author: Tomas Pecka
 */

#ifndef TO_POSTFIX_PUSHDOWN_AUTOMATON_GLUSHKOV_H_
#define TO_POSTFIX_PUSHDOWN_AUTOMATON_GLUSHKOV_H_

#include <automaton/PDA/NPDA.h>

#include <rte/formal/FormalRTE.h>

#include <alib/variant>
#include <alib/set>

#include <alphabet/BottomOfTheStackSymbol.h>

namespace rte {

namespace convert {

/**
 * Converts regular tree expression to real-time height-deterministic pda
 * Source: Master Thesis, Pecka Tomas, CTU FIT, 2016, chapter 4.2
 */
class ToPostfixPushdownAutomatonGlushkov {
private:
	// --------------------------------------------------------------------

	template < class SymbolType, class RankType >
	using TAlphabet = ext::set < common::ranked_symbol < SymbolType, RankType > >;

	// --------------------------------------------------------------------

	template < class SymbolType, class RankType >
	static bool isSubstSymbolPresent ( const ext::set < common::ranked_symbol < SymbolType, RankType > > & container, const TAlphabet < SymbolType, RankType > & substAlphabet );

public:
	/**
	 * Performs conversion.
	 * @param re Original regular tree expression.
	 * @return rhNPDA equivalent to original regular expression.
	 */
	template < class SymbolType, class RankType >
	static automaton::NPDA < SymbolType, DefaultEpsilonType, ext::variant < alphabet::BottomOfTheStackSymbol, ext::set < common::ranked_symbol < SymbolType, RankType > > >, DefaultStateType > convert ( const rte::FormalRTE < > & rte );
};

} /* namespace convert */

} /* namespace rte */

#endif /* TO_POSTFIX_PUSHDOWN_AUTOMATON_GLUSHKOV_V3_H_ */
