/*
 * GlushkovFollowNaive.h
 *
 *  Created on: 14. 4. 2016
 *	  Author: Tomas Pecka
 */

#ifndef RTE_GLUSHKOV_FOLLOW_NAIVE_H_
#define RTE_GLUSHKOV_FOLLOW_NAIVE_H_

#include <alib/map>
#include <alib/set>
#include <alib/vector>
#include <common/ranked_symbol.hpp>

#include <rte/formal/FormalRTE.h>
#include <rte/formal/FormalRTEElements.h>

#include "GlushkovFirst.h"
#include "GlushkovPos.h"

namespace rte {

class GlushkovFollowNaive {

	// --------------------------------------------------------------------

	template < class SymbolType, class RankType >
	using TSubstMap = ext::map < common::ranked_symbol < SymbolType, RankType >, ext::set < common::ranked_symbol < SymbolType, RankType > > >;

	template < class SymbolType, class RankType >
	using TAlphabet = ext::set < common::ranked_symbol < SymbolType, RankType > >;

	// --------------------------------------------------------------------

	template < class SymbolType, class RankType >
	static ext::set < ext::vector < common::ranked_symbol < SymbolType, RankType > > > replaceConstants ( const TAlphabet < SymbolType, RankType > & alphabetK, const ext::vector < ext::set < common::ranked_symbol < SymbolType, RankType > > > & follow, const TSubstMap < SymbolType, RankType > & subMap2 );

	template < class SymbolType, class RankType >
	static void preprocessSubMap ( const TAlphabet < SymbolType, RankType > & alphabetK, TSubstMap < SymbolType, RankType > & subMap );

	template < class SymbolType, class RankType >
	static ext::vector < ext::vector < common::ranked_symbol < SymbolType, RankType > > > cartesian ( const ext::vector < ext::vector < common::ranked_symbol < SymbolType, RankType > > > & input );

	template < class T >
	static void cartesian_rec ( const ext::vector < ext::vector < T > > & input, ext::vector < ext::vector < T > > & ret, ext::vector < T > & current, size_t depth );

public:
	/**
	 * @param re rte to probe
	 * @param symbol FormalRTESymbol for which we need the follow(), i.e., Follow(RTE, symbol)
	 * @return all symbols that can follow specific symbol in word
	 */
	template < class SymbolType, class RankType >
	static ext::set < ext::vector < common::ranked_symbol < SymbolType, RankType > > > follow ( const rte::FormalRTE < SymbolType, RankType > & re, const common::ranked_symbol < SymbolType, RankType > & symbol );

	template < class SymbolType, class RankType >
	class Formal {
	public:
		static ext::set < ext::vector < common::ranked_symbol < SymbolType, RankType > > > visit ( const rte::FormalRTEElement < SymbolType, RankType > & node, const common::ranked_symbol < SymbolType, RankType > & symbol, const TAlphabet < SymbolType, RankType > & alphabetK, TSubstMap < SymbolType, RankType > & subM );
		static ext::set < ext::vector < common::ranked_symbol < SymbolType, RankType > > > visit ( const rte::FormalRTEAlternation < SymbolType, RankType > & node, const common::ranked_symbol < SymbolType, RankType > & symbol, const TAlphabet < SymbolType, RankType > & alphabetK, TSubstMap < SymbolType, RankType > & subM );
		static ext::set < ext::vector < common::ranked_symbol < SymbolType, RankType > > > visit ( const rte::FormalRTESubstitution < SymbolType, RankType > & node, const common::ranked_symbol < SymbolType, RankType > & symbol, const TAlphabet < SymbolType, RankType > & alphabetK, TSubstMap < SymbolType, RankType > & subM );
		static ext::set < ext::vector < common::ranked_symbol < SymbolType, RankType > > > visit ( const rte::FormalRTEIteration < SymbolType, RankType > & node, const common::ranked_symbol < SymbolType, RankType > & symbol, const TAlphabet < SymbolType, RankType > & alphabetK, TSubstMap < SymbolType, RankType > & subM );
		static ext::set < ext::vector < common::ranked_symbol < SymbolType, RankType > > > visit ( const rte::FormalRTESymbolAlphabet < SymbolType, RankType > & node, const common::ranked_symbol < SymbolType, RankType > & symbol, const TAlphabet < SymbolType, RankType > & alphabetK, TSubstMap < SymbolType, RankType > & subM );
		static ext::set < ext::vector < common::ranked_symbol < SymbolType, RankType > > > visit ( const rte::FormalRTESymbolSubst < SymbolType, RankType > & node, const common::ranked_symbol < SymbolType, RankType > & symbol, const TAlphabet < SymbolType, RankType > & alphabetK, TSubstMap < SymbolType, RankType > & subM );
		static ext::set < ext::vector < common::ranked_symbol < SymbolType, RankType > > > visit ( const rte::FormalRTEEmpty < SymbolType, RankType > & node, const common::ranked_symbol < SymbolType, RankType > & symbol, const TAlphabet < SymbolType, RankType > & alphabetK, TSubstMap < SymbolType, RankType > & subM );
	};

};

template < class SymbolType, class RankType >
ext::set < ext::vector < common::ranked_symbol < SymbolType, RankType > > > GlushkovFollowNaive::follow ( const rte::FormalRTE < SymbolType, RankType > & rte, const common::ranked_symbol < SymbolType, RankType > & symbol ) {
	TSubstMap < SymbolType, RankType > subMap;

	 /* Init substitution map, ie \forall a \in K: sub[a] = \emptyset */
	for ( const common::ranked_symbol < SymbolType, RankType > & ssymb : rte.getSubstitutionAlphabet ( ) )
		subMap.insert ( std::make_pair ( ssymb, TAlphabet < SymbolType, RankType > { } ) );

	 /* recursively compute follow */
	return rte.getRTE ( ).getStructure ( ).template accept < ext::set < ext::vector < common::ranked_symbol < SymbolType, RankType > > >, GlushkovFollowNaive::Formal < SymbolType, RankType > > ( symbol, rte.getSubstitutionAlphabet ( ), subMap );
}

// -----------------------------------------------------------------------------

template < class T >
void GlushkovFollowNaive::cartesian_rec ( const ext::vector < ext::vector < T > > & input, ext::vector < ext::vector < T > > & ret, ext::vector < T > & current, size_t depth ) {
	if ( depth == input.size ( ) )
		ret.push_back ( current );
	else
		for ( size_t i = 0; i < input[depth].size ( ); i++ ) {
			current[depth] = input[depth][i];
			cartesian_rec ( input, ret, current, depth + 1 );
		}
}

template < class SymbolType, class RankType >
ext::vector < ext::vector < common::ranked_symbol < SymbolType, RankType > > > GlushkovFollowNaive::cartesian ( const ext::vector < ext::vector < common::ranked_symbol < SymbolType, RankType > > > & input ) {
	ext::vector < ext::vector < common::ranked_symbol < SymbolType, RankType > > > ret;
	ext::vector < common::ranked_symbol < SymbolType, RankType > > current ( input.size ( ), common::ranked_symbol < SymbolType, RankType > ( SymbolType ( 0 ), RankType ( 0 ) ) );

	cartesian_rec ( input, ret, current, 0 );

	return ret;
}

/**
 * Preprocessing:
 *  - Let k1, k2 be elements of alphabet K.
 *  - If k1 is an element of substMap[k2], then copy content of substMap[k1] into substMap[k2]
 */
template < class SymbolType, class RankType >
void GlushkovFollowNaive::preprocessSubMap ( const TAlphabet < SymbolType, RankType > & alphabetK, TSubstMap < SymbolType, RankType > & subMap ) {
	for ( bool change = true; change; change = false )
		for ( std::pair < const common::ranked_symbol < SymbolType, RankType >, TAlphabet < SymbolType, RankType > > & kv : subMap ) {
			TAlphabet < SymbolType, RankType > & substSet = kv.second;

			for ( auto eIter = substSet.begin ( ); eIter != substSet.end ( ); ) {
				if ( alphabetK.count ( * eIter ) == 0 ) {
					++eIter;
				} else {
					auto it = subMap.find ( * eIter );
					size_t oldSize = substSet.size ( );
					substSet.insert ( it->second.begin ( ), it->second.end ( ) );
					change = ( oldSize != substSet.size ( ) ); // something was added
					eIter = substSet.erase ( eIter );
				}
			}
		}
}

template < class SymbolType, class RankType >
ext::set < ext::vector < common::ranked_symbol < SymbolType, RankType > > > GlushkovFollowNaive::replaceConstants ( const TAlphabet < SymbolType, RankType > & alphabetK, const ext::vector < ext::set < common::ranked_symbol < SymbolType, RankType > > > & follow, const TSubstMap < SymbolType, RankType > & subMap2 ) {
	TSubstMap < SymbolType, RankType > subMap ( subMap2 );
	preprocessSubMap ( alphabetK, subMap );

	ext::vector < ext::vector < common::ranked_symbol < SymbolType, RankType > > > input;

	for ( const ext::set < common::ranked_symbol < SymbolType, RankType > > & s : follow ) {
		for ( const common::ranked_symbol < SymbolType, RankType > & e : s ) {
			if ( alphabetK.count ( e ) > 0 )
				input.push_back ( ext::vector < common::ranked_symbol < SymbolType, RankType > > ( subMap.at ( e ).begin ( ), subMap.at ( e ).end ( ) ) );
			else
				input.push_back ( ext::vector < common::ranked_symbol < SymbolType, RankType > > { e } );
		}
	}

	 /* now do the cartesian product on "input" */
	ext::vector < ext::vector < common::ranked_symbol < SymbolType, RankType > > > followSet = cartesian ( input );
	return ext::set < ext::vector < common::ranked_symbol < SymbolType, RankType > > > ( followSet.begin ( ), followSet.end ( ) );
}

// -----------------------------------------------------------------------------

template < class SymbolType, class RankType >
ext::set < ext::vector < common::ranked_symbol < SymbolType, RankType > > > GlushkovFollowNaive::Formal < SymbolType, RankType >::visit ( const rte::FormalRTEAlternation < SymbolType, RankType > & node, const common::ranked_symbol < SymbolType, RankType > & symbolF, const TAlphabet < SymbolType, RankType > & alphabetK, TSubstMap < SymbolType, RankType > & subMap ) {
	ext::set < ext::vector < common::ranked_symbol < SymbolType, RankType > > > ret, tmp;

	tmp = node.getLeftElement ( ).template accept < ext::set < ext::vector < common::ranked_symbol < SymbolType, RankType > > >, GlushkovFollowNaive::Formal < SymbolType, RankType > > ( symbolF, alphabetK, subMap );
	ret.insert ( tmp.begin ( ), tmp.end ( ) );

	tmp = node.getRightElement ( ).template accept < ext::set < ext::vector < common::ranked_symbol < SymbolType, RankType > > >, GlushkovFollowNaive::Formal < SymbolType, RankType > > ( symbolF, alphabetK, subMap );
	ret.insert ( tmp.begin ( ), tmp.end ( ) );

	return ret;
}

template < class SymbolType, class RankType >
ext::set < ext::vector < common::ranked_symbol < SymbolType, RankType > > > GlushkovFollowNaive::Formal < SymbolType, RankType >::visit ( const rte::FormalRTESubstitution < SymbolType, RankType > & node, const common::ranked_symbol < SymbolType, RankType > & symbolF, const TAlphabet < SymbolType, RankType > & alphabetK, TSubstMap < SymbolType, RankType > & subMap ) {

	TSubstMap < SymbolType, RankType > subMap2 ( subMap );
	auto itMap = subMap2.find ( node.getSubstitutionSymbol ( ).getSymbol ( ) );

	itMap->second.clear ( );

	for ( const auto & s : node.getRightElement ( ).template accept < TAlphabet < SymbolType, RankType >, GlushkovFirst::Formal < SymbolType, RankType > > ( ) )
		itMap->second.insert ( s );

	/*
	 * E sub F
	 *   1. if symbolF in F subtree, then Follow(F, symbolF);
	 *   2. if symbolF in E subtree, then Follow(E, symbolF);
	 */

	if ( node.getLeftElement ( ).template accept < bool, GlushkovPos::Formal < SymbolType, RankType > > ( symbolF ) )
		return node.getLeftElement ( ).template accept < ext::set < ext::vector < common::ranked_symbol < SymbolType, RankType > > >, GlushkovFollowNaive::Formal < SymbolType, RankType > > ( symbolF, alphabetK, subMap2 );
	else
		return node.getRightElement ( ).template accept < ext::set < ext::vector < common::ranked_symbol < SymbolType, RankType > > >, GlushkovFollowNaive::Formal < SymbolType, RankType > > ( symbolF, alphabetK, subMap );
}

template < class SymbolType, class RankType >
ext::set < ext::vector < common::ranked_symbol < SymbolType, RankType > > > GlushkovFollowNaive::Formal < SymbolType, RankType >::visit ( const rte::FormalRTEIteration < SymbolType, RankType > & node, const common::ranked_symbol < SymbolType, RankType > & symbolF, const TAlphabet < SymbolType, RankType > & alphabetK, TSubstMap < SymbolType, RankType > & subMap ) {

	ext::set < ext::vector < common::ranked_symbol < SymbolType, RankType > > > ret;

	for ( const auto & s : node.getElement ( ).template accept < TAlphabet < SymbolType, RankType >, GlushkovFirst::Formal < SymbolType, RankType > > ( ) )
		subMap[node.getSubstitutionSymbol ( ).getSymbol ( )].insert ( s );

	return node.getElement ( ).template accept < ext::set < ext::vector < common::ranked_symbol < SymbolType, RankType > > >, GlushkovFollowNaive::Formal < SymbolType, RankType > > ( symbolF, alphabetK, subMap );
}

template < class SymbolType, class RankType >
ext::set < ext::vector < common::ranked_symbol < SymbolType, RankType > > > GlushkovFollowNaive::Formal < SymbolType, RankType >::visit ( const rte::FormalRTESymbolAlphabet < SymbolType, RankType > & node, const common::ranked_symbol < SymbolType, RankType > & symbolF, const TAlphabet < SymbolType, RankType > & alphabetK, TSubstMap < SymbolType, RankType > & subMap ) {

	ext::set < ext::vector < common::ranked_symbol < SymbolType, RankType > > > ret, tmp;

	if ( symbolF == node.getSymbol ( ) ) {
		ext::vector < ext::set < common::ranked_symbol < SymbolType, RankType > > > children;

		for ( const rte::FormalRTEElement < SymbolType, RankType > & c : node.getElements ( ) )
			children.push_back ( c.template accept < TAlphabet < SymbolType, RankType >, GlushkovFirst::Formal < SymbolType, RankType > > ( ) );

		return replaceConstants ( alphabetK, children, subMap );
	}

	for ( const auto & c : node.getElements ( ) ) {
		tmp = c.template accept < ext::set < ext::vector < common::ranked_symbol < SymbolType, RankType > > >, GlushkovFollowNaive::Formal < SymbolType, RankType > > ( symbolF, alphabetK, subMap );
		ret.insert ( tmp.begin ( ), tmp.end ( ) );
	}

	return ret;
}

template < class SymbolType, class RankType >
ext::set < ext::vector < common::ranked_symbol < SymbolType, RankType > > > GlushkovFollowNaive::Formal < SymbolType, RankType >::visit ( const rte::FormalRTESymbolSubst < SymbolType, RankType > & /* node */, const common::ranked_symbol < SymbolType, RankType > & /* symbolF */, const TAlphabet < SymbolType, RankType > & /* alphabetK */, TSubstMap < SymbolType, RankType > & /* subMap */ ) {
	return ext::set < ext::vector < common::ranked_symbol < SymbolType, RankType > > > ( );
}

template < class SymbolType, class RankType >
ext::set < ext::vector < common::ranked_symbol < SymbolType, RankType > > > GlushkovFollowNaive::Formal < SymbolType, RankType >::visit ( const rte::FormalRTEEmpty < SymbolType, RankType > & /* node */, const common::ranked_symbol < SymbolType, RankType > & /* symbolF */, const TAlphabet < SymbolType, RankType > & /* alphabetK */, TSubstMap < SymbolType, RankType > & /* subMap */ ) {
	return ext::set < ext::vector < common::ranked_symbol < SymbolType, RankType > > > ( );
}

} /* namespace rte */

#endif /* RTE_GLUSHKOV_FOLLOW_NAIVE_H_ */
