/*
 * GlushkovPos.h
 *
 *  Created on: 14. 4. 2016
 *	  Author: Tomas Pecka
 */

#ifndef RTE_GLUSHKOV_POS_H_
#define RTE_GLUSHKOV_POS_H_

#include <common/ranked_symbol.hpp>

#include <rte/formal/FormalRTE.h>
#include <rte/formal/FormalRTEElements.h>

namespace rte {

class GlushkovPos {
public:
	/**
	 * @return bool true if symbol pointer is in this subtree
	 */
	template < class SymbolType, class RankType >
	static bool pos ( const common::ranked_symbol < SymbolType, RankType > & symbol, const rte::FormalRTE < SymbolType, RankType > & node );

	template < class SymbolType, class RankType >
	class Formal {
	public:
		static bool visit ( const rte::FormalRTEAlternation < SymbolType, RankType > & node, const common::ranked_symbol < SymbolType, RankType > & symbSearch );
		static bool visit ( const rte::FormalRTESubstitution < SymbolType, RankType > & node, const common::ranked_symbol < SymbolType, RankType > & symbSearch );
		static bool visit ( const rte::FormalRTEIteration < SymbolType, RankType > & node, const common::ranked_symbol < SymbolType, RankType > & symbSearch );
		static bool visit ( const rte::FormalRTESymbolAlphabet < SymbolType, RankType > & node, const common::ranked_symbol < SymbolType, RankType > & symbSearch );
		static bool visit ( const rte::FormalRTESymbolSubst < SymbolType, RankType > & node, const common::ranked_symbol < SymbolType, RankType > & symbSearch );
		static bool visit ( const rte::FormalRTEEmpty < SymbolType, RankType > & node, const common::ranked_symbol < SymbolType, RankType > & symbSearch );
	};
};

template < class SymbolType, class RankType >
bool GlushkovPos::pos ( const common::ranked_symbol < SymbolType, RankType > & symbol, const rte::FormalRTE < SymbolType, RankType > & rte ) {
	return rte.getRTE ( ).getStructure ( ).template accept < bool, GlushkovPos::Formal < SymbolType, RankType > > ( symbol );
}

template < class SymbolType, class RankType >
bool GlushkovPos::Formal < SymbolType, RankType >::visit ( const rte::FormalRTEAlternation < SymbolType, RankType > & node, const common::ranked_symbol < SymbolType, RankType > & symbolF ) {
	return node.getLeftElement ( ).template accept < bool, GlushkovPos::Formal < SymbolType, RankType > > ( symbolF ) || node.getRightElement ( ).template accept < bool, GlushkovPos::Formal < SymbolType, RankType > > ( symbolF );
}

template < class SymbolType, class RankType >
bool GlushkovPos::Formal < SymbolType, RankType >::visit ( const rte::FormalRTESubstitution < SymbolType, RankType > & node, const common::ranked_symbol < SymbolType, RankType > & symbolF ) {
	return node.getLeftElement ( ).template accept < bool, GlushkovPos::Formal < SymbolType, RankType > > ( symbolF ) || node.getRightElement ( ).template accept < bool, GlushkovPos::Formal < SymbolType, RankType > > ( symbolF );
}

template < class SymbolType, class RankType >
bool GlushkovPos::Formal < SymbolType, RankType >::visit ( const rte::FormalRTEIteration < SymbolType, RankType > & node, const common::ranked_symbol < SymbolType, RankType > & symbolF ) {
	return node.getElement ( ).template accept < bool, GlushkovPos::Formal < SymbolType, RankType > > ( symbolF );
}

template < class SymbolType, class RankType >
bool GlushkovPos::Formal < SymbolType, RankType >::visit ( const rte::FormalRTESymbolAlphabet < SymbolType, RankType > & node, const common::ranked_symbol < SymbolType, RankType > & symbolF ) {
	if ( symbolF == node.getSymbol ( ) ) return true;

	for ( const rte::FormalRTEElement < SymbolType, RankType > & element : node.getElements ( ) )
		if ( element.template accept < bool, GlushkovPos::Formal < SymbolType, RankType > > ( symbolF ) )
			return true;

	return false;
}

template < class SymbolType, class RankType >
bool GlushkovPos::Formal < SymbolType, RankType >::visit ( const rte::FormalRTESymbolSubst < SymbolType, RankType > & node, const common::ranked_symbol < SymbolType, RankType > & symbolF ) {
	return symbolF == node.getSymbol ( );
}

template < class SymbolType, class RankType >
bool GlushkovPos::Formal < SymbolType, RankType >::visit ( const rte::FormalRTEEmpty < SymbolType, RankType > & /* node */, const common::ranked_symbol < SymbolType, RankType > & /* symbolF */ ) {
	return false;
}

} /* namespace rte */

#endif /* RTE_GLUSHKOV_POS_H_ */
