/*
 * FormalRTE.h
 *
 * Created on: Dec 1, 2017
 * Author: Jan Travnicek
 */

#ifndef _XML_FORMAL_RTE_H_
#define _XML_FORMAL_RTE_H_

#include <rte/formal/FormalRTE.h>
#include <rte/xml/FormalRTEStructure.h>
#include <core/xmlApi.hpp>

#include <rte/xml/common/RTEFromXmlParser.h>
#include <rte/xml/common/RTEToXmlComposer.h>

namespace core {

template < typename SymbolType, class RankType >
struct xmlApi < rte::FormalRTE < SymbolType, RankType > > {
	static rte::FormalRTE < SymbolType, RankType > parse ( ext::deque < sax::Token >::iterator & input );
	static bool first ( const ext::deque < sax::Token >::const_iterator & input );
	static const std::string & xmlTagName ( );
	static void compose ( ext::deque < sax::Token > & output, const rte::FormalRTE < SymbolType, RankType > & data );
};

template < typename SymbolType, class RankType >
rte::FormalRTE < SymbolType, RankType > xmlApi < rte::FormalRTE < SymbolType, RankType > >::parse ( ext::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, xmlTagName ( ) );

	std::pair < ext::set < common::ranked_symbol < SymbolType, RankType > >, ext::set < common::ranked_symbol < SymbolType, RankType > > > alphabets = rte::RTEFromXmlParser::parseAlphabet < SymbolType, RankType > ( input );
	rte::FormalRTEStructure < SymbolType, RankType > element ( core::xmlApi < rte::FormalRTEStructure < SymbolType, RankType > >::parse ( input ) );
	rte::FormalRTE < SymbolType, RankType > rte ( std::move ( alphabets.first ), std::move ( alphabets.second ), std::move ( element ) );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, xmlTagName ( ) );
	return rte;
}

template < typename SymbolType, class RankType >
bool xmlApi < rte::FormalRTE < SymbolType, RankType > >::first ( const ext::deque < sax::Token >::const_iterator & input ) {
	return sax::FromXMLParserHelper::isToken ( input, sax::Token::TokenType::START_ELEMENT, xmlTagName ( ) );
}

template < typename SymbolType, class RankType >
const std::string & xmlApi < rte::FormalRTE < SymbolType, RankType > >::xmlTagName ( ) {
	static std::string xmlTagName = "FormalRTE";

	return xmlTagName;
}

template < typename SymbolType, class RankType >
void xmlApi < rte::FormalRTE < SymbolType, RankType > >::compose ( ext::deque < sax::Token > & output, const rte::FormalRTE < SymbolType, RankType > & input ) {
	output.emplace_back ( xmlTagName ( ), sax::Token::TokenType::START_ELEMENT );
	rte::RTEToXmlComposer::composeAlphabet ( output, input.getAlphabet ( ), input.getSubstitutionAlphabet ( ) );
	core::xmlApi < rte::FormalRTEStructure < SymbolType, RankType > >::compose ( output, input.getRTE ( ) );
	output.emplace_back ( xmlTagName ( ), sax::Token::TokenType::END_ELEMENT );
}

} /* namespace core */

#endif /* _XML_FORMAL_RTE_H_ */
