/*
 * RTEFromXmlParser.h
 *
 *  Created on: Feb 20, 2016
 *      Author: Tomas Pecka
 */

#ifndef RTE_FROM_XML_PARSER_H_
#define RTE_FROM_XML_PARSER_H_

#include <alib/set>
#include <alib/deque>
#include <sax/Token.h>
#include <common/ranked_symbol.hpp>
#include <core/xmlApi.hpp>

#include <exception/CommonException.h>

#include <rte/formal/FormalRTEElements.h>


namespace rte {

/**
 * Parser used to get RTE from XML parsed into list of tokens.
 */
class RTEFromXmlParser {
public:
	template < class SymbolType, class RankType >
	static std::pair < ext::set < common::ranked_symbol < SymbolType, RankType > >, ext::set < common::ranked_symbol < SymbolType, RankType > > > parseAlphabet ( ext::deque < sax::Token >::iterator & input );

	template < class SymbolType, class RankType >
	static ext::ptr_value < FormalRTEElement < SymbolType, RankType > > parseFormalRTEElement ( ext::deque < sax::Token >::iterator & input );

	template < class SymbolType, class RankType >
	static ext::ptr_value < FormalRTEEmpty < SymbolType, RankType > > parseFormalRTEEmpty ( ext::deque < sax::Token >::iterator & input );
	template < class SymbolType, class RankType >
	static ext::ptr_value < FormalRTEIteration < SymbolType, RankType > > parseFormalRTEIteration ( ext::deque < sax::Token >::iterator & input );
	template < class SymbolType, class RankType >
	static ext::ptr_value < FormalRTEAlternation < SymbolType, RankType > > parseFormalRTEAlternation ( ext::deque < sax::Token >::iterator & input );
	template < class SymbolType, class RankType >
	static ext::ptr_value < FormalRTESubstitution < SymbolType, RankType > > parseFormalRTESubstitution ( ext::deque < sax::Token >::iterator & input );
	template < class SymbolType, class RankType >
	static ext::ptr_value < FormalRTESymbolAlphabet < SymbolType, RankType > > parseFormalRTESymbolAlphabet ( ext::deque < sax::Token >::iterator & input );
	template < class SymbolType, class RankType >
	static ext::ptr_value < FormalRTESymbolSubst < SymbolType, RankType > > parseFormalRTESymbolSubst ( ext::deque < sax::Token >::iterator & input );
};

template < class SymbolType, class RankType >
std::pair < ext::set < common::ranked_symbol < SymbolType, RankType > >, ext::set < common::ranked_symbol < SymbolType, RankType > > > RTEFromXmlParser::parseAlphabet ( ext::deque < sax::Token >::iterator & input ) {
	ext::set < common::ranked_symbol < SymbolType, RankType > > alphabetF, alphabetK;

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "alphabet" );

	while ( sax::FromXMLParserHelper::isTokenType ( input, sax::Token::TokenType::START_ELEMENT ) )
		alphabetF.insert ( core::xmlApi < common::ranked_symbol < SymbolType, RankType > >::parse ( input ) );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "alphabet" );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "substSymbolAlphabet" );

	while ( sax::FromXMLParserHelper::isTokenType ( input, sax::Token::TokenType::START_ELEMENT ) )
		alphabetK.insert ( core::xmlApi < common::ranked_symbol < SymbolType, RankType > >::parse ( input ) );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "substSymbolAlphabet" );

	return std::make_pair ( alphabetF, alphabetK );
}

template < class SymbolType, class RankType >
ext::ptr_value < FormalRTEElement < SymbolType, RankType > > RTEFromXmlParser::parseFormalRTEElement ( ext::deque < sax::Token >::iterator & input ) {
	if ( sax::FromXMLParserHelper::isToken ( input, sax::Token::TokenType::START_ELEMENT, "empty" ) )
		return parseFormalRTEEmpty < SymbolType, RankType > ( input );
	else if ( sax::FromXMLParserHelper::isToken ( input, sax::Token::TokenType::START_ELEMENT, "iteration" ) )
		return parseFormalRTEIteration < SymbolType, RankType > ( input );
	else if ( sax::FromXMLParserHelper::isToken ( input, sax::Token::TokenType::START_ELEMENT, "alternation" ) )
		return parseFormalRTEAlternation < SymbolType, RankType > ( input );
	else if ( sax::FromXMLParserHelper::isToken ( input, sax::Token::TokenType::START_ELEMENT, "substitution" ) )
		return parseFormalRTESubstitution < SymbolType, RankType > ( input );
	else if ( sax::FromXMLParserHelper::isToken ( input, sax::Token::TokenType::START_ELEMENT, "symbol" ) )
		return parseFormalRTESymbolAlphabet < SymbolType, RankType > ( input );
	else
		return parseFormalRTESymbolSubst < SymbolType, RankType > ( input );
}

template < class SymbolType, class RankType >
ext::ptr_value < FormalRTEAlternation < SymbolType, RankType > > RTEFromXmlParser::parseFormalRTEAlternation ( ext::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "alternation" );

	ext::ptr_value < FormalRTEElement < SymbolType, RankType > > element1 = parseFormalRTEElement < SymbolType, RankType > ( input );
	ext::ptr_value < FormalRTEElement < SymbolType, RankType > > element2 = parseFormalRTEElement < SymbolType, RankType > ( input );

	ext::ptr_value < FormalRTEAlternation < SymbolType, RankType > > alternation ( FormalRTEAlternation < SymbolType, RankType > ( std::move ( element1 ), std::move ( element2 ) ) );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "alternation" );
	return alternation;
}

template < class SymbolType, class RankType >
ext::ptr_value < FormalRTESubstitution < SymbolType, RankType > > RTEFromXmlParser::parseFormalRTESubstitution ( ext::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "substitution" );

	FormalRTESymbolSubst < SymbolType, RankType > ssymb ( core::xmlApi < common::ranked_symbol < SymbolType, RankType > >::parse ( input ) );
	ext::ptr_value < FormalRTEElement < SymbolType, RankType > > element1 = parseFormalRTEElement < SymbolType, RankType > ( input );
	ext::ptr_value < FormalRTEElement < SymbolType, RankType > > element2 = parseFormalRTEElement < SymbolType, RankType > ( input );

	ext::ptr_value < FormalRTESubstitution < SymbolType, RankType > > substitution ( FormalRTESubstitution < SymbolType, RankType > ( std::move ( element1 ), std::move ( element2 ), std::move ( ssymb ) ) );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "substitution" );
	return substitution;
}

template < class SymbolType, class RankType >
ext::ptr_value < FormalRTEIteration < SymbolType, RankType > > RTEFromXmlParser::parseFormalRTEIteration ( ext::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "iteration" );

	FormalRTESymbolSubst < SymbolType, RankType > ssymb ( core::xmlApi < common::ranked_symbol < SymbolType, RankType > >::parse ( input ) );
	ext::ptr_value < FormalRTEElement < SymbolType, RankType > > element = parseFormalRTEElement < SymbolType, RankType > ( input );
	ext::ptr_value < FormalRTEIteration < SymbolType, RankType > > iteration ( FormalRTEIteration < SymbolType, RankType > ( std::move ( element ), std::move ( ssymb ) ) );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "iteration" );
	return iteration;
}

template < class SymbolType, class RankType >
ext::ptr_value < FormalRTEEmpty < SymbolType, RankType > > RTEFromXmlParser::parseFormalRTEEmpty ( ext::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "empty" );

	ext::ptr_value < FormalRTEEmpty < SymbolType, RankType > > empty { FormalRTEEmpty < SymbolType, RankType > ( ) };

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "empty" );

	return empty;
}

template < class SymbolType, class RankType >
ext::ptr_value < FormalRTESymbolAlphabet < SymbolType, RankType > > RTEFromXmlParser::parseFormalRTESymbolAlphabet ( ext::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "symbol" );
	common::ranked_symbol < SymbolType, RankType > symbol = core::xmlApi < common::ranked_symbol < SymbolType, RankType > >::parse ( input );
	ext::ptr_vector < FormalRTEElement < SymbolType, RankType > > elements;

	while ( sax::FromXMLParserHelper::isTokenType ( input, sax::Token::TokenType::START_ELEMENT ) )
		elements.push_back ( ( FormalRTEElement < SymbolType, RankType > && ) parseFormalRTEElement < SymbolType, RankType > ( input ) );

	ext::ptr_value < FormalRTESymbolAlphabet < SymbolType, RankType > > ret ( FormalRTESymbolAlphabet < SymbolType, RankType > ( std::move ( symbol ), std::move ( elements ) ) );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "symbol" );

	return ret;
}

template < class SymbolType, class RankType >
ext::ptr_value < FormalRTESymbolSubst < SymbolType, RankType > > RTEFromXmlParser::parseFormalRTESymbolSubst ( ext::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "substSymbol" );
	common::ranked_symbol < SymbolType, RankType > symbol = core::xmlApi < common::ranked_symbol < SymbolType, RankType > >::parse ( input );

	ext::ptr_value < FormalRTESymbolSubst < SymbolType, RankType > > ret { FormalRTESymbolSubst < SymbolType, RankType > ( symbol ) };

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "substSymbol" );

	return ret;
}

} /* namespace rte */

#endif /* RTE_FROM_XML_PARSER_H_ */
