/*
 * RTEToXmlComposer.h
 *
 *  Created on: Feb 20, 2016
 *      Author: Tomas Pecka
 */

#ifndef RTE_TO_XML_COMPOSER_H_
#define RTE_TO_XML_COMPOSER_H_

#include <alib/deque>
#include <rte/formal/FormalRTEElement.h>
#include <sax/Token.h>
#include <core/xmlApi.hpp>

#include <alphabet/xml/RankedSymbol.h>

namespace rte {

/**
 * This class contains methods to print XML representation of regular tree expression to the output stream.
 */
class RTEToXmlComposer {
public:
	template < class SymbolType, class RankType >
	static void composeAlphabet ( ext::deque < sax::Token > & out, const ext::set < common::ranked_symbol < SymbolType, RankType > > & alphabetF, const ext::set < common::ranked_symbol < SymbolType, RankType > > & alphabetK );

	class Formal {
	public:
		template < class SymbolType, class RankType >
		static void visit ( const FormalRTEAlternation < SymbolType, RankType > & alternation, ext::deque < sax::Token > & output );
		template < class SymbolType, class RankType >
		static void visit ( const FormalRTEIteration < SymbolType, RankType > & iteration, ext::deque < sax::Token > & output );
		template < class SymbolType, class RankType >
		static void visit ( const FormalRTESubstitution < SymbolType, RankType > & concatenation, ext::deque < sax::Token > & output );
		template < class SymbolType, class RankType >
		static void visit ( const FormalRTESymbolAlphabet < SymbolType, RankType > & symbol, ext::deque < sax::Token > & output );
		template < class SymbolType, class RankType >
		static void visit ( const FormalRTESymbolSubst < SymbolType, RankType > & symbol, ext::deque < sax::Token > & output );
		template < class SymbolType, class RankType >
		static void visit ( const FormalRTEEmpty < SymbolType, RankType > & empty, ext::deque < sax::Token > & output );
	};
};

template < class SymbolType, class RankType >
void RTEToXmlComposer::Formal::visit ( const FormalRTEAlternation < SymbolType, RankType > & alternation, ext::deque < sax::Token > & out ) {
	out.emplace_back ( "alternation", sax::Token::TokenType::START_ELEMENT );
	alternation.getLeftElement ( ).template accept < void, RTEToXmlComposer::Formal > ( out );
	alternation.getRightElement ( ).template accept < void, RTEToXmlComposer::Formal > ( out );
	out.emplace_back ( "alternation", sax::Token::TokenType::END_ELEMENT );
}

template < class SymbolType, class RankType >
void RTEToXmlComposer::Formal::visit ( const FormalRTESubstitution < SymbolType, RankType > & substitution, ext::deque < sax::Token > & out ) {
	out.emplace_back ( "substitution", sax::Token::TokenType::START_ELEMENT );

	core::xmlApi < common::ranked_symbol < SymbolType, RankType > >::compose ( out, substitution.getSubstitutionSymbol ( ).getSymbol ( ) );
	substitution.getLeftElement ( ).template accept < void, RTEToXmlComposer::Formal > ( out );
	substitution.getRightElement ( ).template accept < void, RTEToXmlComposer::Formal > ( out );

	out.emplace_back ( "substitution", sax::Token::TokenType::END_ELEMENT );
}

template < class SymbolType, class RankType >
void RTEToXmlComposer::Formal::visit ( const FormalRTEIteration < SymbolType, RankType > & iteration, ext::deque < sax::Token > & out ) {
	out.emplace_back ( "iteration", sax::Token::TokenType::START_ELEMENT );

	core::xmlApi < common::ranked_symbol < SymbolType, RankType > >::compose ( out, iteration.getSubstitutionSymbol ( ).getSymbol ( ) );
	iteration.getElement ( ).template accept < void, RTEToXmlComposer::Formal > ( out );

	out.emplace_back ( "iteration", sax::Token::TokenType::END_ELEMENT );
}

template < class SymbolType, class RankType >
void RTEToXmlComposer::Formal::visit ( const FormalRTESymbolAlphabet < SymbolType, RankType > & symbol, ext::deque < sax::Token > & out ) {
	out.emplace_back ( sax::Token ( "symbol", sax::Token::TokenType::START_ELEMENT ) );
	core::xmlApi < common::ranked_symbol < SymbolType, RankType > >::compose ( out, symbol.getSymbol ( ) );

	for ( const FormalRTEElement < SymbolType, RankType > & element : symbol.getElements ( ) )
		element.template accept < void, RTEToXmlComposer::Formal > ( out );

	out.emplace_back ( sax::Token ( "symbol", sax::Token::TokenType::END_ELEMENT ) );
}

template < class SymbolType, class RankType >
void RTEToXmlComposer::Formal::visit ( const FormalRTESymbolSubst < SymbolType, RankType > & symbol, ext::deque < sax::Token > & out ) {
	out.emplace_back ( sax::Token ( "substSymbol", sax::Token::TokenType::START_ELEMENT ) );
	core::xmlApi < common::ranked_symbol < SymbolType, RankType > >::compose ( out, symbol.getSymbol ( ) );

	out.emplace_back ( sax::Token ( "substSymbol", sax::Token::TokenType::END_ELEMENT ) );
}

template < class SymbolType, class RankType >
void RTEToXmlComposer::Formal::visit ( const FormalRTEEmpty < SymbolType, RankType > &, ext::deque < sax::Token > & out ) {
	out.emplace_back ( "empty", sax::Token::TokenType::START_ELEMENT );
	out.emplace_back ( "empty", sax::Token::TokenType::END_ELEMENT );
}

template < class SymbolType, class RankType >
void RTEToXmlComposer::composeAlphabet ( ext::deque < sax::Token > & out, const ext::set < common::ranked_symbol < SymbolType, RankType > > & alphabetF, const ext::set < common::ranked_symbol < SymbolType, RankType > > & alphabetK ) {
	out.emplace_back ( "alphabet", sax::Token::TokenType::START_ELEMENT );

	for ( const auto & symbol : alphabetF )
		core::xmlApi < common::ranked_symbol < SymbolType, RankType > >::compose ( out, symbol );

	out.emplace_back ( "alphabet", sax::Token::TokenType::END_ELEMENT );

	out.emplace_back ( "substSymbolAlphabet", sax::Token::TokenType::START_ELEMENT );

	for ( const auto & symbol : alphabetK )
		core::xmlApi < common::ranked_symbol < SymbolType, RankType > >::compose ( out, symbol );

	out.emplace_back ( "substSymbolAlphabet", sax::Token::TokenType::END_ELEMENT );
}

} /* namespace rte */

#endif /* RTE_TO_XML_COMPOSER_H_ */
