/*
 * ExactCompare.h
 *
 *  Created on: Oct 14, 2014
 *	  Author: Radomir Polach
 */

#ifndef EXACT_COMPARE_H_
#define EXACT_COMPARE_H_

#include <string/LinearString.h>
#include <string/CyclicString.h>
#include <string/Epsilon.h>

namespace string {

namespace naive {

class ExactCompare {
public:
	template < class SymbolType >
	static int compare(const string::Epsilon < SymbolType >& u, const string::Epsilon < SymbolType >& v);
	template < class SymbolType >
	static int compare(const string::LinearString < SymbolType >& u, const string::LinearString < SymbolType >& v);
	template < class SymbolType >
	static int compare(const string::CyclicString < SymbolType >& u, const string::CyclicString < SymbolType >& v);
};

template < class SymbolType >
int ExactCompare::compare ( const string::Epsilon < SymbolType > &, const string::Epsilon < SymbolType > & ) {
	return 0;
}

template < class SymbolType >
int ExactCompare::compare ( const string::LinearString < SymbolType > & u, const string::LinearString < SymbolType > & v ) {
	int n = ( int ) u.getContent ( ).size ( ), m = ( int ) v.getContent ( ).size ( );
	int k = 0;

	while ( k < n && k < m && u.getContent ( )[k] == v.getContent ( )[k] ) k++;

	if ( ( k == m ) && ( k == n ) )
		return 0;
	else if ( k == m )
		return -1;
	else if ( k == n )
		return 1;
	else if ( u.getContent ( )[k] < v.getContent ( )[k] )
		return -1;
	else
		return 1;
}

template < class SymbolType >
int ExactCompare::compare ( const string::CyclicString < SymbolType > & u, const string::CyclicString < SymbolType > & v ) {
	int n = ( int ) u.getContent ( ).size ( ), m = ( int ) v.getContent ( ).size ( );
	int i = -1, j = -1, k;

	bool last = 0;

	while ( i < n - 1 && j < m - 1 ) {
		k = 1;

		while ( k <= n && u.getContent ( )[( i + k ) % n] == v.getContent ( )[( j + k ) % m] ) k++;

		if ( k > n ) return 0;

		last = u.getContent ( )[( i + k ) % n] > v.getContent ( )[( j + k ) % m];

		if ( last )
			i += k;
		else
			j += k;
	}

	return last ? 1 : -1;
}

} /* namespace naive */

} /* namespace string */

#endif /* EXACT_COMPARE_H_ */
