/*
 * ExactEqual.h
 *
 *  Created on: Oct 14, 2014
 *	  Author: Radomir Polach
 */

#ifndef EXACT_EQUAL_H_
#define EXACT_EQUAL_H_

#include <string/LinearString.h>
#include <string/CyclicString.h>
#include <string/Epsilon.h>

namespace string {

namespace naive {

class ExactEqual {
public:
	template < class SymbolType >
	static bool equals(const string::Epsilon < SymbolType >& u, const string::Epsilon < SymbolType >& v);
	template < class SymbolType >
	static bool equals(const string::LinearString < SymbolType >& u, const string::LinearString < SymbolType >& v);
	template < class SymbolType >
	static bool equals(const string::CyclicString < SymbolType >& u, const string::CyclicString < SymbolType >& v);
};

template < class SymbolType >
bool ExactEqual::equals ( const string::Epsilon < SymbolType > &, const string::Epsilon < SymbolType > & ) {
	return true;
}

template < class SymbolType >
bool ExactEqual::equals ( const string::LinearString < SymbolType > & u, const string::LinearString < SymbolType > & v ) {
	int n = ( int ) u.getContent ( ).size ( ), m = ( int ) v.getContent ( ).size ( );
	int k = 0;

	while ( k < n && k < m && u.getContent ( )[k] == v.getContent ( )[k] ) k++;

	if ( ( k == m ) && ( k == n ) )
		return true;
	else
		return false;
}

template < class SymbolType >
bool ExactEqual::equals ( const string::CyclicString < SymbolType > & u, const string::CyclicString < SymbolType > & v ) {
	int n = ( int ) u.getContent ( ).size ( );
	int i = -1, j = -1, k;

	if ( n != ( int ) v.getContent ( ).size ( ) ) return false;

	while ( i < n - 1 && j < n - 1 ) {
		k = 1;

		while ( k <= n && u.getContent ( )[( i + k ) % n] == v.getContent ( )[( j + k ) % n] ) k++;

		if ( k > n ) return true;

		if ( u.getContent ( )[( i + k ) % n] > v.getContent ( )[( j + k ) % n] )
			i += k;
		else
			j += k;
	}

	return false;
}

} /* namespace naive */

} /* namespace string */

#endif /* EXACT_EQUAL_H_ */
