/*
 * NormalizeAlphabet.h
 *
 *  Created on: Dec 9, 2013
 *	  Author: Jan Travnicek
 */

#ifndef NORMALIZE_ALPHABET_H_
#define NORMALIZE_ALPHABET_H_

#include <string/LinearString.h>

namespace string {

namespace simplify {

class NormalizeAlphabet {
public:
	/**
	 * @param dfa automaton to normalize
	 */
	template < class SymbolType >
	static string::LinearString < SymbolType > normalize(const string::LinearString < SymbolType > & str);
};

template < class SymbolType >
string::LinearString < SymbolType > NormalizeAlphabet::normalize(const string::LinearString < SymbolType > & string) {
	int counter = 0;
	ext::map<SymbolType, char > normalizationData;

	for(const SymbolType& symbol : string.getContent()) {
		if(normalizationData.find(symbol) == normalizationData.end()) {
			normalizationData.insert(std::make_pair(symbol, counter++));
		}
	}

	ext::set<SymbolType> alphabet;
	for(const auto& symbol : normalizationData) {
		alphabet.insert(SymbolType((char) (symbol.second + 'a')));
	}

	ext::vector<SymbolType> data;
	for(const SymbolType& symbol : string.getContent()) {
		data.push_back(SymbolType((char) (normalizationData.find(symbol)->second + 'a')));
	}

	string::LinearString < > result(alphabet, data);
	return result;
}

} /* namespace simplify */

} /* namespace string */

#endif /* NORMALIZE_ALPHABET_H_ */
