/*
 * RankedNonlinearPattern.h
 *
 * Created on: Sep 27, 2017
 * Author: Jan Travnicek
 */

#ifndef _STRING_RANKED_PATTERN_H_
#define _STRING_RANKED_PATTERN_H_

#include <tree/ranked/RankedNonlinearPattern.h>
#include <core/stringApi.hpp>

#include <tree/TreeFromStringLexer.h>

#include <tree/string/common/TreeFromStringParserCommon.h>
#include <tree/string/common/TreeToStringComposerCommon.h>

namespace core {

template<class SymbolType, class RankType >
struct stringApi < tree::RankedNonlinearPattern < SymbolType, RankType > > {
	static tree::RankedNonlinearPattern < SymbolType, RankType > parse ( std::istream & input );
	static bool first ( std::istream & input );
	static void compose ( std::ostream & output, const tree::RankedNonlinearPattern < SymbolType, RankType > & tree );
};

template<class SymbolType, class RankType >
tree::RankedNonlinearPattern < SymbolType, RankType > stringApi < tree::RankedNonlinearPattern < SymbolType, RankType > >::parse ( std::istream & input ) {
	tree::TreeFromStringLexer::Token token = tree::TreeFromStringLexer::next ( input );
	if ( token.type != tree::TreeFromStringLexer::TokenType::RANKED_NONLINEAR_PATTERN )
		throw exception::CommonException ( "Unrecognised RANKED_NONLINEAR_PATTERN token." );

	ext::set < common::ranked_symbol < SymbolType, RankType > > nonlinearVariables;
	bool isPattern = false;

	ext::tree < common::ranked_symbol < SymbolType, RankType > > content = tree::TreeFromStringParserCommon::parseRankedContent < SymbolType, RankType > ( input, isPattern, nonlinearVariables );
	if ( ! isPattern && ! nonlinearVariables.size ( ) )
		throw exception::CommonException ( "Invalid input" );

	return tree::RankedNonlinearPattern < SymbolType, RankType > ( alphabet::WildcardSymbol::instance < common::ranked_symbol < SymbolType > > ( ), nonlinearVariables, content );
}

template<class SymbolType, class RankType >
bool stringApi < tree::RankedNonlinearPattern < SymbolType, RankType > >::first ( std::istream & input ) {
	tree::TreeFromStringLexer::Token token = tree::TreeFromStringLexer::next ( input );
	bool res = token.type == tree::TreeFromStringLexer::TokenType::RANKED_NONLINEAR_PATTERN;
	tree::TreeFromStringLexer::putback ( input, token );
	return res;
}

template<class SymbolType, class RankType >
void stringApi < tree::RankedNonlinearPattern < SymbolType, RankType > >::compose ( std::ostream &, const tree::RankedNonlinearPattern < SymbolType, RankType > & ) {
	throw exception::CommonException ( "Unimplemented." );
}

} /* namespace core */

#endif /* _STRING_RANKED_PATTERN_H_ */
