/*
 * RankedPattern.h
 *
 * Created on: Sep 27, 2017
 * Author: Jan Travnicek
 */

#ifndef _STRING_RANKED_PATTERN_H_
#define _STRING_RANKED_PATTERN_H_

#include <tree/ranked/RankedPattern.h>
#include <core/stringApi.hpp>

#include <tree/TreeFromStringLexer.h>

#include <tree/string/common/TreeFromStringParserCommon.h>
#include <tree/string/common/TreeToStringComposerCommon.h>

namespace core {

template<class SymbolType, class RankType >
struct stringApi < tree::RankedPattern < SymbolType, RankType > > {
	static tree::RankedPattern < SymbolType, RankType > parse ( std::istream & input );
	static bool first ( std::istream & input );
	static void compose ( std::ostream & output, const tree::RankedPattern < SymbolType, RankType > & tree );
};

template<class SymbolType, class RankType >
tree::RankedPattern < SymbolType, RankType > stringApi < tree::RankedPattern < SymbolType, RankType > >::parse ( std::istream & input ) {
	tree::TreeFromStringLexer::Token token = tree::TreeFromStringLexer::next ( input );
	if ( token.type != tree::TreeFromStringLexer::TokenType::RANKED_PATTERN )
		throw exception::CommonException ( "Unrecognised RANKED_PATTERN token." );

	ext::set < common::ranked_symbol < SymbolType, RankType > > nonlinearVariables;
	bool isPattern = false;

	ext::tree < common::ranked_symbol < SymbolType, RankType > > content = tree::TreeFromStringParserCommon::parseRankedContent < SymbolType, RankType > ( input, isPattern, nonlinearVariables );
	if ( ! isPattern || nonlinearVariables.size ( ) )
		throw exception::CommonException ( "Invalid input" );

	return tree::RankedPattern < SymbolType, RankType > ( alphabet::WildcardSymbol::instance < common::ranked_symbol < SymbolType > > ( ), content );
}

template<class SymbolType, class RankType >
bool stringApi < tree::RankedPattern < SymbolType, RankType > >::first ( std::istream & input ) {
	tree::TreeFromStringLexer::Token token = tree::TreeFromStringLexer::next ( input );
	bool res = token.type == tree::TreeFromStringLexer::TokenType::RANKED_PATTERN;
	tree::TreeFromStringLexer::putback ( input, token );
	return res;
}

template<class SymbolType, class RankType >
void stringApi < tree::RankedPattern < SymbolType, RankType > >::compose ( std::ostream & output, const tree::RankedPattern < SymbolType, RankType > & tree ) {
	output << "RANKED_PATTERN ";
	tree::TreeToStringComposerCommon::compose ( output, tree.getSubtreeWildcard ( ), tree.getContent ( ) );
}

} /* namespace core */

#endif /* _STRING_RANKED_PATTERN_H_ */
